from zs_utils.api.ebay.base_api import EbayAPI


class FindEligibleItems(EbayAPI):
    """
    Docs:
    https://developer.ebay.com/api-docs/sell/negotiation/resources/offer/methods/findEligibleItems
    """

    http_method = "GET"
    resource_method = "sell/negotiation/v1/find_eligible_items"
    allowed_params = ["limit", "offset"]
    MAX_LIMIT = 200


class SendOfferToInterestedBuyers(EbayAPI):
    """
    Docs:
    https://developer.ebay.com/api-docs/sell/negotiation/resources/offer/methods/sendOfferToInterestedBuyers
    """

    http_method = "POST"
    resource_method = "sell/negotiation/v1/send_offer_to_interested_buyers"



# import argparse
# import os

# from pysftp import CnOpts, Connection

# cnopts = CnOpts()
# cnopts.hostkeys = None

# credentials = {
#     "host": "mip.ebay.com",
#     "port": 22,
#     "username": "zone-smart",
#     "password": "v^1.1#i^1#r^1#I^3#f^0#p^3#t^Ul4xMF82OjQ5REY2QzJDQjM2RTVERDE5MDIyRTAwNDU3NUQ3MzVBXzNfMSNFXjI2MA==",
# }

# mip_dirs = [
#     "availability",
#     "deleteInventory",
#     "distribution",
#     "inventory",
#     "location",
#     "order",
#     "orderfulfillment",
#     "product",
#     "reports",
# ]

# parser = argparse.ArgumentParser(description="Загрузка eBay feed")


# class StoreNameValuePair(argparse.Action):
#     def __call__(self, parser, namespace, values, option_string=None):
#         n, v = values.split("=")
#         setattr(namespace, n, v)


# parser.add_argument("file_path", action=StoreNameValuePair, help="Путь к feed-файлу на локальной машине")
# parser.add_argument(
#     "mip_dir",
#     action=StoreNameValuePair,
#     help="Папка на сервере mip.ebay.com, в которую будет загружен feed-файл",
# )
# # parser.add_argument('--apps', nargs='+', required=False,
# #                    help='список приложений, в которых будут очищаться папки migrations')
# args = parser.parse_args()

# file_path = args.file_path
# if not os.path.isfile(file_path):
#     raise AttributeError(f"Файл {file_path} не существует на локальной машине!")

# mip_dir = args.mip_dir
# if mip_dir not in mip_dirs:
#     raise AttributeError(f"Недопустимая папка для загрузки: {mip_dir}\n" f"Допустимые папки: {mip_dirs}")


# def callback(ready, total):
#     print(f"File was {'un' if ready!=total else ''}successfully uploaded:")
#     print(f"{ready} of {total} bytes uploaded")


# with Connection(**credentials, cnopts=cnopts) as sftp:
#     sftp.chdir(os.path.join("store", mip_dir))
#     try:
#         sftp.put(localpath=file_path, callback=callback)
#     except FileNotFoundError as err:
#         print(err)



from .abstract import AbstractModel, XMLElement

from zs_utils.yandex_market_language.exceptions import ValidationError


CONDITION_CHOICES = ("likenew", "used")


class Condition(AbstractModel):
    """
    Condition model.
    Used for used goods and goods discounted due to deficiencies.

    Docs:
    https://yandex.ru/support/partnermarket/elements/condition.html
    """

    __slots__ = ["_condition_type", "reason"]

    def __init__(self, condition_type: str, reason: str):
        self.condition_type = condition_type
        self.reason = reason

    @property
    def condition_type(self) -> str:
        return self._condition_type

    @condition_type.setter
    def condition_type(self, value):
        if value not in CONDITION_CHOICES:
            raise ValidationError(
                "condition_type attribute must be a value from a list: "
                "{list}".format(list=", ".join(CONDITION_CHOICES))
            )
        self._condition_type = value

    def create_dict(self, **kwargs) -> dict:
        return dict(condition_type=self.condition_type, reason=self.reason)

    def create_xml(self, **kwargs) -> XMLElement:
        el = XMLElement("condition", {"type": self.condition_type})
        reason_el = XMLElement("reason")
        reason_el.text = self.reason
        el.append(reason_el)
        return el

    @staticmethod
    def from_xml(el: XMLElement) -> "Condition":
        if el:
            reason = el[0].text.lower()
        else:
            reason = None
        return Condition(condition_type=el.attrib["type"], reason=reason)



from datetime import datetime, timedelta

from zs_utils.api.ebay.base_api import EbayTradingAPI
from zs_utils.api.ebay.data.enums import PlatformNotificationEventTypeEnum


__all__ = [
    "GetNotificationSettings",
    "GetAppNotificationSettings",
    "GetUserNotificationSettings",
    "GetNotificationsUsage",
    "SetNotificationSettings",
    "ReviseNotifications",
    "SubscribeNotification",
    "UnSubscribeNotification",
]


class GetNotificationSettings(EbayTradingAPI):
    """
    Docs:
    https://developer.ebay.com/Devzone/XML/docs/Reference/eBay/GetNotificationPreferences.html
    """

    method_name = "GetNotificationPreferences"

    def get_params(self, preference_level: str, **kwargs):
        assert preference_level in [
            "Application",
            "Event",
            "User",
            "UserData",
        ], f'Недопустимое значение "preference_level": {preference_level}'
        return {
            "PreferenceLevel": preference_level,
            "OutputSelector": None,
        }

    # DEPRECATED
    # def make_request(self, **kwargs):
    #     is_success, message, objects = super().make_request(**kwargs)
    #     if objects.get("errors", []):
    #         if objects["errors"][0].get("ErrorCode", None) == "12209":
    #             is_success = True
    #             objects["results"] = []
    #     return is_success, message, objects


class GetAppNotificationSettings(GetNotificationSettings):
    def get_params(self, **kwargs):
        kwargs["preference_level"] = "Application"
        return super().get_params(**kwargs)


class GetUserNotificationSettings(GetNotificationSettings):
    def get_params(self, **kwargs):
        kwargs["preference_level"] = "User"
        return super().get_params(**kwargs)


class GetNotificationsUsage(EbayTradingAPI):
    """
    Docs:
    https://developer.ebay.com/Devzone/XML/docs/Reference/eBay/GetNotificationsUsage.html
    FIX: timeout problem
    """

    method_name = "GetNotificationsUsage"

    def get_params(
        self,
        remote_id: str = None,
        start_time: str = None,
        end_time: str = None,
        hours_ago: int = None,
        **kwargs,
    ):
        if hours_ago:
            start_time = (datetime.now() - timedelta(hours=hours_ago)).isoformat()
            end_time = datetime.now().isoformat()
        elif not end_time:
            end_time = datetime.now().isoformat()

        return {
            "ItemID": remote_id,
            "StartTime": start_time,
            "EndTime": end_time,
        }


class SetNotificationSettings(EbayTradingAPI):
    """
    Docs:
    https://developer.ebay.com/Devzone/XML/docs/Reference/eBay/SetNotificationPreferences.html
    """

    method_name = "SetNotificationPreferences"

    def get_params(
        self,
        alert_enable: bool = None,
        app_enable: bool = None,
        alert_email: str = None,
        app_url: str = None,
        delivery_urls: dict = None,
        subscriptions: list = None,
        user_identifier: str = None,
        **kwargs,
    ):
        params = {
            "ApplicationDeliveryPreferences": {
                "AlertEmail": alert_email,
                "ApplicationURL": app_url,
                "DeviceType": "Platform",
            },
            "UserDeliveryPreferenceArray": subscriptions,
        }

        if app_enable is not None:
            params["ApplicationDeliveryPreferences"]["ApplicationEnable"] = "Enable" if app_enable else "Disable"

        if alert_enable is not None:
            params["ApplicationDeliveryPreferences"]["AlertEnable"] = "Enable" if alert_enable else "Disable"

        if user_identifier:
            params["UserData"] = {
                "ExternalUserData": user_identifier,
            }

        if delivery_urls:
            params["ApplicationDeliveryPreferences"].update(
                {
                    "DeliveryURLDetails": [
                        {
                            "DeliveryURL": url,
                            "DeliveryURLName": url,
                            "Status": "Enable" if enable else "Disable",
                        }
                        for url, enable in delivery_urls.items()
                    ]
                }
            )
            params["DeliveryURLName"] = ",".join(list(delivery_urls.keys()))

        return params


class ReviseNotifications(SetNotificationSettings):
    def get_params(self, notifications: list, enable: bool, **kwargs):
        for notification in notifications:
            assert (
                notification in PlatformNotificationEventTypeEnum
            ), f'Недопустимое значение "notification": {notification}'

        kwargs["subscriptions"] = [
            {
                "NotificationEnable": [
                    {
                        "EventType": notification,
                        "EventEnable": "Enable" if enable else "Disable",
                    }
                    for notification in notifications
                ]
            }
        ]
        return super().get_params(**kwargs)


class SubscribeNotification(ReviseNotifications):
    def get_params(self, **kwargs):
        kwargs["enable"] = True
        return super().get_params(**kwargs)


class UnSubscribeNotification(ReviseNotifications):
    def get_params(self, **kwargs):
        kwargs["enable"] = False
        return super().get_params(**kwargs)



from model_utils import Choices


CancelStateEnum = Choices(
    ("CANCELED", "Заказ отменён"),
    ("IN_PROGRESS", "Сделан хотя бы один запрос на отмену заказа"),
    ("NONE_REQUESTED", "Запросов на отмену заказа нет"),
)

CancelRequestStateEnum = Choices(
    ("COMPLETED", "Продавец подтвердил отмену заказа"),
    ("REJECTED", "Продавец отказал в отмене заказа"),
    ("REQUESTED", "Запрос на отмену заказа ожидает ответа от продавца"),
)

FulfillmentInstructionsType = Choices(
    ("DIGITAL", "Цифровой вид"),
    ("PREPARE_FOR_PICKUP", "Готовится к In-Store Pickup"),
    ("SELLER_DEFINED", "Определяется продавцом"),
    ("SHIP_TO", "Отправляется продавцом"),
)

LineItemFulfillmentStatusEnum = Choices(
    ("FULFILLED", "Фулфилмент завершен"),
    ("IN_PROGRESS", "Фулфилмент в процессе"),
    ("NOT_STARTED", "Фулфилмент не начат"),
)

SoldFormatEnum = Choices(
    ("AUCTION", "Аукцион"),
    ("FIXED_PRICE", "Фиксированная цена"),
    ("OTHER", "Другое"),
    ("SECOND_CHANCE_OFFER", "Second chance offer"),
)

OrderFulfillmentStatus = Choices(
    ("FULFILLED", "Фулфилмент завершен"),
    ("IN_PROGRESS", "Фулфилмент в процессе"),
    ("NOT_STARTED", "Фулфилмент не начат"),
)

OrderPaymentStatusEnum = Choices(
    ("FAILED", "Неудача"),
    ("FULLY_REFUNDED", "Деньги в полном объеме возвращены покупателю"),
    ("PAID", "Оплачено"),
    ("PARTIALLY_REFUNDED", "Деньги частично возвращены покупателю"),
    ("PENDING", "Ожидание"),
)

PaymentMethodTypeEnum = Choices(("CREDIT_CARD", "Банковская карта"), ("PAYPAL", "Paypal"))

PaymentStatusEnum = Choices(
    ("FAILED", "Неудача"),
    ("PAID", "Оплачено"),
    ("PENDING", "Ожидание"),
)

PaymentHoldStateEnum = Choices(
    ("HELD", "Заморожено"),
    ("HELD_PENDING", "Ожидается заморозка"),
    ("NOT_HELD", "Не заморожено"),
    ("RELEASE_CONFIRMED", "Подтверждено размораживание"),
    ("RELEASE_FAILED", "Неудачное размораживание"),
    ("RELEASE_PENDING", "Ожидается размораживание"),
    ("RELEASED", "Разморожено"),
)

RefundStatusEnum = Choices(
    ("FAILED", "Неудача"),
    ("PENDING", "Ожидание"),
    ("REFUNDED", "Деньги возвращены"),
)

TaxTypeEnum = Choices(
    ("GST", "Goods and Services import tax"),
    ("PROVINCE_SALES_TAX", "Provincial sales tax"),
    ("REGION", "Regional sales tax"),
    ("STATE_SALES_TAX", "State sales tax"),
    ("VAT", "Value-Added tax (VAT)"),
)

DisputeStateEnum = Choices(
    ("OPEN", "Открыто"),
    ("ACTION_NEEDED", "Требуется действие"),
    ("CLOSED", "Закрыто"),
)



from zs_utils.api.ozon.base_api import OzonAPI


class OzonGetCategoriesTreeAPI(OzonAPI):
    """
    Docs: https://docs.ozon.ru/api/seller/#operation/DescriptionCategoryAPI_GetTree
    """

    resource_method = "v1/description-category/tree"
    allowed_params = ["language"]


class OzonGetCategoryAttributeAPI(OzonAPI):
    """
    Docs: https://docs.ozon.ru/api/seller/#operation/DescriptionCategoryAPI_GetAttributes
    """

    resource_method = "v1/description-category/attribute"
    required_params = ["description_category_id", "type_id"]
    allowed_params = ["language"]


class OzonGetCategoryAttributeDictionaryAPI(OzonAPI):
    """
    Docs: https://docs.ozon.ru/api/seller/#operation/DescriptionCategoryAPI_GetAttributeValues
    """

    resource_method = "v1/description-category/attribute/values"
    required_params = ["attribute_id", "description_category_id", "limit", "type_id"]
    allowed_params = ["language", "last_value_id"]



from model_utils import Choices

ShippingCarriersEnum = Choices(
    ("Other", "Другое"),
    ("DHL", "DHL Express"),
    ("DHLEKB", "DHL EKB"),
    ("DHLEXPRESS", "DHL Express"),
    ("DHLGlobalMail", "DHL Global Mail"),
    ("UPS", "United Parcel Service"),
    ("USPS", "U.S. Postal Service"),
    ("GENERIC", "Generic"),
)

AllShippingCarriersEnum = Choices(
    ("Other", "Use this code for any carrier not listed here."),
    ("A1CourierServices", "A-1 Courier"),
    ("ABF", "ABF Freight"),
    ("AeroPost", "AeroPost"),
    ("ALLIEDEXPRESS", "Allied Express"),
    ("AMWST", "AMWST"),
    ("AnPost", "An Post"),
    ("APC", "APC Postal Logistics"),
    ("ARAMEX", "Aramex"),
    ("ARVATO", "Arvato"),
    ("ASM", "ASM"),
    ("AustralianAirExpress", "Australian Air Express"),
    ("AustraliaPost", "Australia Post"),
    ("AVRT", "Averitt Express"),
    ("Bartolini", "BRT Bartolini"),
    ("BELGIANPOST", "Belgian Post Group"),
    ("BKNS", "BKNS"),
    ("BluePackage", "Blue Package Delivery"),
    ("BPost", "bpost"),
    ("BusinessPost", "BusinessPost"),
    ("CanPar", "Canpar Courier"),
    ("CENF", "Central Freight Lines"),
    ("CEVA", "CEVA Logistics"),
    ("ChinaPost", "China Post"),
    ("Chronoexpres", "Chronoexpres"),
    ("Chronopost", "Chronopost"),
    ("CHUKOU1", "Chukou1"),
    ("ChunghwaPost", "Chunghwa Post"),
    ("CitiPost", "CitiPost"),
    ("CityLink", "Citylink"),
    ("ClickandQuick", "Click & Quick"),
    ("CNWY", "XPO Logistics (formerly Con-way Freight)"),
    ("ColiposteDomestic", "Coliposte Domestic"),
    ("ColiposteInternational", "Coliposte International"),
    ("Colissimo", "Colissimo"),
    ("CollectPlus", "CollectPlus"),
    ("Correos", "Correos"),
    ("CPC", "CPC Logistics"),
    ("DAIPost", "DAI Post"),
    ("DayandRoss", "Day & Ross"),
    ("DBSchenker", "DB Schenker"),
    ("DeutschePost", "Deutsche Post"),
    ("DHL", "DHL Express"),
    ("DHLEKB", "DHL EKB"),
    ("DHLEXPRESS", "DHL Express"),
    ("DHLGlobalMail", "DHL Global Mail"),
    ("DieSchweizerischePost", "Die Schweizerische Post"),
    ("DPD", "DPD (Dynamic Parcel Distribution)"),
    ("DPXThailand", "DPX Thailand"),
    ("EGO", "E-go"),
    ("Exapaq", "DPD France (formerly Exapaq)"),
    ("Fastway", "Fastway"),
    ("FASTWAYCOURIERS", "Fastway Couriers"),
    ("FedEx", "FedEx"),
    ("FedExSmartPost", "FedEx SmartPost"),
    ("FLYT", "Flyt"),
    ("FLYTExpress", "Flyt Express"),
    ("FlytExpressUSDirectline", "Flyt Express US Direct line"),
    ("FourPX", "4PX"),
    ("FourPXCHINA", "4PX China"),
    ("FourPXExpress", "4PX Express"),
    ("FourPXLTD", "4PX Express Co. Ltd"),
    ("FulfilExpressAccStation", "FulfilExpress-AccStation"),
    ("FulfilExpresseForCity", "FulfilExpress-eForCity"),
    ("FulfilExpressEverydaySource", "FulfilExpress-EverydaySource"),
    ("FulfilExpressiTrimming", "FulfilExpress-iTrimming"),
    ("GLS", "GLS (General Logistics Systems)"),
    ("HDUSA", "MXD Group (formerly Home Direct USA)"),
    ("Hermes", "Hermes Group"),
    ("HongKongPost", "Hong Kong Post"),
    ("HUNTEREXPRESS", "Hunter Express"),
    ("iLoxx", "iloxx eService"),
    ("IndiaPost", "India Post"),
    ("IndonesiaPost", "Indonesia Post"),
    ("Interlink", "Interlink Express"),
    ("InterPost", "InterPost"),
    ("IoInvio", "IoInvio"),
    ("Iparcel", "UPS i-parcel"),
    ("IsraelPost", "Israel Post"),
    ("JapanPost", "Japan Post"),
    ("KIALA", "Kiala (UPS Access Point)"),
    ("KoreaPost", "Korea Post"),
    ("Landmark", "Landmark Global"),
    ("LAPOSTE", "La Poste"),
    ("MALAYSIAPOST", "Malaysia Post"),
    ("MannaFreight", "Manna Distribution Services"),
    ("Metapack", "Metapack"),
    ("MNGTurkey", "MNG Kargo"),
    ("MondialRelay", "Mondial Relay"),
    ("MRW", "MRW"),
    ("MSI", "MSI Transportation"),
    ("Nacex", "Nacex"),
    ("NEMF", "New England Motor Freight"),
    ("ODFL", "Old Dominion Freight Line"),
    ("ONTRACK", "OnTrac Shipping"),
    ("OsterreichischePostAG", "Osterreichische Post"),
    ("Parcelforce", "Parcelforce"),
    ("ParcelPool", "International Bridge Domestic delivery"),
    ("Philpost", "PHLPost (Philippine Postal Corporation)"),
    ("Pilot", "Pilot Freight Services"),
    ("PITD", "PITT OHIO"),
    ("PocztaPolska", "Poczta Polska"),
    ("Pocztex", "Pocztex"),
    ("PosteItaliane", "Poste Italiane"),
    ("POSTITALIANO", "Post Italiano"),
    ("PostNL", "PostNL"),
    ("PostNordNorway", "PostNord"),
    ("Quantium", "Quantium Solutions"),
    ("RETL", "Reddaway"),
    ("RoyalMail", "Royal Mail"),
    ("SAIA", "Saia LTL Freight"),
    ("SDA", "SDA Express Courier"),
    ("SINGAPOREPOST", "Singapore Post"),
    ("Siodemka", "Siodemka (DPD Poland)"),
    ("SioliandFontana", "Sioli & Fontana"),
    ("SkynetMalaysia", "Skynet (Malaysia)"),
    ("SMARTSEND", "Smart Send Courier Service"),
    ("Sogetras", "SGT Corriere Espresso"),
    ("Spediamo", "Spediamo"),
    ("SpeeDee", "Spee-Dee Delivery Service"),
    ("StarTrack", "StarTrack"),
    ("SuntekExpressLTD", "Suntek Express LTD"),
    ("SwissPost", "Swiss Post"),
    ("TELE", "TELE"),
    ("TEMANDO", "Temando (shipping broker)"),
    ("THAILANDPOST", "Thailand Post"),
    ("Toll", "Toll (Japan Post)"),
    ("TPG", "TPG Logistics"),
    ("UBI", "UBI Smart Parcel"),
    ("UKMail", "UK Mail"),
    ("UPS", "United Parcel Service"),
    ("USPS", "U.S. Postal Service"),
    ("USPSCeP", "USPS Commercial ePacket"),
    ("USPSPMI", "USPS Priority Mail International"),
    ("VietnamPost", "Vietnam Post"),
    ("VITR", "Vitran Express"),
    ("Winit", "WIN.IT America"),
    ("WNdirect", "wnDirect"),
    ("WPX", "WPX Delivery Solutions"),
    ("YANWEN", "YANWEN Express"),
    ("Yodel", "Yodel"),
    ("YRC", "YRC Freight"),
)



from zs_utils.api.shopify.base_api import ShopifyAPI


class UpdateShopifyInventoryAPI(ShopifyAPI):
    """
    https://shopify.dev/api/admin-rest/2022-01/resources/inventorylevel#post-inventory-levels-set
    """

    http_method = "POST"
    resource_method = "inventory_levels/set.json"
    required_params = [
        "available",
        "inventory_item_id",
        "location_id",
    ]
    allowed_params = ["disconnect_if_necessary"]


class GetShopifyInventoryLevelListAPI(ShopifyAPI):
    """
    https://shopify.dev/docs/api/admin-rest/2023-10/resources/inventorylevel#get-inventory-levels?location-ids=655441491
    """

    http_method = "GET"
    resource_method = "inventory_levels.json"
    required_params = []
    allowed_params = [
        "inventory_item_ids",  # <= 50
        "limit",  # <= 250
        "location_ids",  # <= 50
        "updated_at_min",
        "page_info",
    ]



from zs_utils.api.ebay.base_api import EbayAPI


class PostOrderAPI(EbayAPI):
    @property
    def headers(self):
        return {
            "Authorization": f"IAF {self.access_token}",
            "Content-Type": "application/json",
            "Accept": "application/json",
            "X-EBAY-C-MARKETPLACE-ID": self.marketplace_id,
        }


class SearchCancellations(PostOrderAPI):
    """
    Docs: https://developer.ebay.com/Devzone/post-order/post-order_v2_cancellation_search__get.html
    """

    resource_method = "post-order/v2/cancellation/search"
    http_method = "GET"
    allowed_params = [
        "limit",
        "offset",
        "role",
        "creation_date_range_from",
        "creation_date_range_to",
        "cancel_id",
        "item_id",
        "legacy_order_id",
        "transaction_id",
        "sort",
    ]


class SearchReturns(PostOrderAPI):
    """
    Docs: https://developer.ebay.com/devzone/post-order/post-order_v2_return_search__get.html
    """

    resource_method = "post-order/v2/return/search"
    http_method = "GET"
    allowed_params = [
        "limit",
        "offset",
        "creation_date_range_from",
        "creation_date_range_to",
        "role",
        "item_id",
        "order_id",
        "return_id",
        "return_state",
        "sort",
        "states",
        "transaction_id",
    ]



import requests

from django.conf import settings
from django.utils.translation import gettext as _

from zs_utils.exceptions import CaptchaException


def validate_captcha(token: str, ip: str) -> None:
    resp = requests.get(
        url="https://captcha-api.yandex.ru/validate",
        params={
            "secret": settings.YANDEX_CAPTCHA_SERVER_KEY,
            "token": token,
            "ip": ip,
        },
        timeout=1,
    )
    resp.raise_for_status()

    results = resp.json()
    if results["status"] != "ok":
        raise CaptchaException(message=_("Ошибка валидации капчи: {error}").format(error=results["message"]))



from rest_framework import status
from rest_framework.response import Response
from rest_framework.exceptions import ValidationError
from rest_framework.views import exception_handler as drf_exception_handler

from zs_utils.json_utils import pretty_json


__all__ = [
    "CustomException",
    "CaptchaException",
]


class CustomException(Exception):
    status_code: int = status.HTTP_400_BAD_REQUEST

    message: str = None
    messages: list = None
    message_dict: dict = None
    message_dict_key_prefix: str = None

    response = None
    response_code = None

    reason = None

    def __init__(
        self,
        message: str = None,
        *,
        instance=None,
        messages: list = None,
        message_dict: dict = None,
        message_dict_key_prefix: str = None,
        status_code=None,
        response=None,
        reason=None,
        **kwargs,
    ):
        super().__init__(**kwargs)

        attrs = dict(
            message=message,
            messages=messages,
            message_dict=message_dict,
            message_dict_key_prefix=message_dict_key_prefix,
            status_code=status_code,
            response=response,
            reason=reason,
        )

        if instance:
            if not isinstance(instance, self.__class__):
                raise ValueError(f"Параметр 'instance' должен быть экземпляром класса '{self.__class__.__name__}'.")

            for key, value in attrs.items():
                if value is None:
                    attrs[key] = getattr(instance, key)

        self.set_attributes(**attrs)

    def __str__(self):
        message = ""

        if self.message:
            message += str(self.message)
        elif self.messages:
            message += "; ".join(self.messages)

        if self.message_dict:
            message += "\n" + pretty_json(data=self.message_dict)

        if (not message) and (self.response_code):
            message = f"Code: {self.response_code}"

        return message

    def __repr__(self) -> str:
        val = ""
        if self.messages:
            val = f"messages={self.messages}"
        elif self.message:
            val = f"message={self.message}"
        if self.message_dict:
            if val:
                val += ", "
            val += f"message_dict={self.message_dict}"
        return f"{self.__class__.__name__}({val})"

    def set_attributes(self, **kwargs):
        for key, value in kwargs.items():
            if hasattr(self, key):
                if key == "status_code":
                    if not value:
                        continue
                elif key == "response":
                    if value is not None:
                        for field in ["code", "status_code", "status"]:
                            if getattr(value, field, None):
                                self.response_code = getattr(value, field)
                                break
                elif key == "message_dict":
                    prefix = kwargs.get("message_dict_key_prefix")
                    if prefix:
                        value = self.add_key_prefix(data=value, prefix=prefix)

                setattr(self, key, value)

    def add_key_prefix(self, data: dict, prefix: str, connector: str = ".", safe: bool = True):
        updated_data = {}

        for key, value in data.items():
            if (not safe) or (not key.startswith(prefix)):
                updated_key = f"{prefix}{connector}{key}"
            else:
                updated_key = str(key)

            if isinstance(value, dict):
                updated_data[updated_key] = self.add_key_prefix(data=value, prefix=prefix, connector=connector)
            else:
                updated_data[updated_key] = value

        return updated_data

    def to_dict(self):
        if self.message_dict:
            data = dict(self.message_dict)
        elif self.messages:
            data = {"messages": self.messages}
        elif self.message:
            data = {"message": self.message}
        else:
            data = {"message": super().__str__()}

        return data

    def message_dict_to_str(self) -> str:
        if self.message_dict:
            return "\n".join(f"{key}: {value}" for key, value in self.message_dict.items())
        else:
            return ""

    def messages_to_str(self) -> str:
        if self.messages:
            if len(self.messages) == 1:
                return str(self.messages[0])
            else:
                return "\n".join(f"{num}) {msg}" for num, msg in enumerate(self.messages, start=1))
        else:
            return ""


class CaptchaException(CustomException):
    pass


def flatten_errors_dict(data=None, parent_string=None):
    result = {}

    if isinstance(data, dict):
        for key, value in data.items():
            if isinstance(value, dict) or isinstance(value, list):
                result.update(
                    flatten_errors_dict(
                        data=value,
                        parent_string=parent_string + "." + str(key) if parent_string else str(key),
                    )
                )
            else:
                pre_parent_string = parent_string + "." if parent_string else ""
                result[pre_parent_string + str(key)] = value
    elif isinstance(data, list):
        for item in data:
            if isinstance(item, dict):
                for num, item in enumerate(data):
                    result.update(flatten_errors_dict(data=item, parent_string=f"{parent_string}.{num}"))
                break
        else:
            result[parent_string] = data

    return result


def custom_exception_handler(exc: Exception, context: dict) -> Response:
    """
    Кастомный обработчик ошибок.
    """

    if isinstance(exc, CustomException):
        if (not exc.status_code) or (not status.is_server_error(code=exc.status_code)):
            detail = {}

            if exc.message:
                detail["message"] = exc.message

            if exc.messages:
                detail["messages"] = exc.messages

            if exc.message_dict:
                detail["errors"] = exc.message_dict

            exc = ValidationError(detail=detail, code=exc.status_code)
    elif isinstance(exc, ValidationError) and isinstance(exc.detail, dict):
        exc.detail = flatten_errors_dict(data=exc.detail)
        if "errors" not in exc.detail:
            exc.detail = {"errors": exc.detail}

    # if settings.SENTRY_ENABLED:
    #     capture_sentry_exception(error=exc, user=context.get("user"))

    return drf_exception_handler(exc, context)



from zs_utils.api.base_api import BaseAPI
from zs_utils.api.yandex_market.base_api import YandexMarketAPI


class GetYandexMarketShopListAPI(YandexMarketAPI):
    """
    https://yandex.ru/dev/market/partner-dsbs/doc/dg/reference/get-campaigns.html
    """

    description = "Получение списка магазинов Yandex Market"
    http_method = "GET"
    resource_method = "campaigns"
    allowed_params = ["page", "pageSize"]


class GetYandexMarketShopAPI(YandexMarketAPI):
    """
    https://yandex.ru/dev/market/partner-dsbs/doc/dg/reference/get-campaigns-id.html
    """

    description = "Получение информации о магазине Yandex Market"
    http_method = "GET"
    resource_method = "campaigns/{shop_id}"
    required_params = [
        "shop_id",
    ]


class GetYandexMarketShopLoginListAPI(YandexMarketAPI):
    """
    https://yandex.ru/dev/market/partner-dsbs/doc/dg/reference/get-campaigns-id-logins.html
    """

    description = "Получение списка логинов, у которых есть доступ к магазину Yandex Market"
    http_method = "GET"
    resource_method = "campaigns/{shop_id}/logins"
    required_params = [
        "shop_id",
    ]


class GetYandexMarketShopListByLoginAPI(YandexMarketAPI):
    """
    https://yandex.ru/dev/market/partner-dsbs/doc/dg/reference/get-campaigns-by-login.html
    """

    description = "Получение списка магазинов, к которым есть доступ по данному логину. Yandex Market"
    http_method = "GET"
    resource_method = "campaigns/by_login/{login}"
    required_params = [
        "login",
    ]


class GetYandexMarketShopSettingsAPI(YandexMarketAPI):
    """
    https://yandex.ru/dev/market/partner-dsbs/doc/dg/reference/get-campaigns-id-settings.html
    """

    description = "Получение настроек магазина Yandex Market"
    http_method = "GET"
    resource_method = "campaigns/{shop_id}/settings"
    required_params = [
        "shop_id",
    ]


class GetYandexMarketShopCategoriesAPI(YandexMarketAPI):
    """
    https://yandex.ru/dev/market/partner-dsbs/doc/dg/reference/get-campaigns-id-feeds-categories.html
    """

    description = "Получение категорий магазина Яндекс.Маркета"
    http_method = "GET"
    resource_method = "campaigns/{shop_id}/feeds/categories"
    required_params = [
        "shop_id",
    ]


class GetYandexMarketOutletsAPI(YandexMarketAPI):
    """
    https://yandex.ru/dev/market/partner-dsbs/doc/dg/reference/get-campaigns-id-outlets.html
    """

    description = "Получение точек продаж магазина Яндекс.Маркета"
    http_method = "GET"
    resource_method = "campaigns/{shop_id}/outlets"
    required_params = [
        "shop_id",
    ]


class GetYandexMarketAccountInfo(BaseAPI):
    http_method = "GET"

    def __init__(self, access_token: str, **kwargs):
        super().__init__(**kwargs)

        self.access_token = access_token

    def build_url(self, params: dict) -> str:
        return f"https://login.yandex.ru/info?format=json&oauth_token={self.access_token}"

    def validate_attrs(self) -> None:
        pass



from zs_utils.api.insales.base_api import InsalesAPI


class GetInsalesPaymentPolicyList(InsalesAPI):
    """
    https://api.insales.ru/#paymentgateway-get-payment-gateways-json
    """

    resource_method = "payment_gateways.json"
    http_method = "GET"


class GetInsalesDeliveryPolicyList(InsalesAPI):
    """
    https://api.insales.ru/#deliveryvariant-get-delivery-variants-json
    """

    resource_method = "delivery_variants.json"
    http_method = "GET"



import inspect


__all__ = [
    "get_params_for_func",
    "call_func_with_filtered_params",
]


def _get_func_params(func) -> dict:
    """
    Получение информации о принимаемых функцией (или методом класса) func параметрах из сигнатуры.
    """

    params = {}
    for name, info in dict(inspect.signature(func).parameters).items():
        data = {"kind": info.kind.name}
        if info.default != info.empty:
            data["default"] = info.default
        params[name] = data
    return params


def get_params_for_func(func, data: dict) -> dict:
    """
    Получение параметров для вызова функции func из словаря data.
    """

    params = {"kwargs": {}}

    for param_name, info in _get_func_params(func=func).items():
        if info["kind"] == "VAR_POSITIONAL":
            continue

        if param_name in data:
            value = data[param_name]
        elif "default" in info:
            value = info["default"]
        elif info["kind"] != "VAR_KEYWORD":
            raise ValueError(f"В словаре 'data' отсутствует обязательный ключ '{param_name}'.")
        else:
            continue

        if info["kind"] == "VAR_KEYWORD":
            if not isinstance(value, dict):
                raise ValueError(f"Параметр '{param_name}' должен быть словарем.")
            params["kwargs"] = value
        else:
            params[param_name] = value

    return params


def call_func_with_filtered_params(func, data: dict):
    """
    Вызов функции func с параметрами, полученными из словаря data.
    """

    params: dict = get_params_for_func(func=func, data=data)
    params.update(params.pop("kwargs", {}))
    return func(**params)



from typing import List

from zs_utils.yandex_market_language import models, exceptions
from zs_utils.yandex_market_language.models import fields
from zs_utils.yandex_market_language.models.abstract import XMLElement, XMLSubElement

from zs_utils.yandex_market_language.exceptions import ValidationError


class Shop(
    fields.EnableAutoDiscountField, fields.DeliveryOptionsField, fields.PickupOptionsField, models.AbstractModel
):
    """
    Shop model.

    Docs:
    https://yandex.ru/support/partnermarket/elements/shop.html
    """

    __slots__ = [
        "_url",
        "name",
        "company",
        "currencies",
        "categories",
        "offers",
        "platform",
        "version",
        "agency",
        "email",
        "_delivery_options",
        "_pickup_options",
        "_enable_auto_discounts",
        "gifts",
        "promos",
    ]

    def __init__(
        self,
        offers: List["models.offers.AbstractOffer"] = None,
        name: str = None,
        company: str = None,
        url: str = None,
        currencies: List["models.Currency"] = None,
        categories: List["models.Category"] = None,
        platform: str = None,
        version: str = None,
        agency: str = None,
        email: str = None,
        delivery_options: List["models.Option"] = None,
        pickup_options: List["models.Option"] = None,
        enable_auto_discounts=None,
        gifts: List["models.Gift"] = None,
        promos: List["models.Promo"] = None,
        **kwargs,
    ):
        if not offers:
            offers = []

        if not url:
            url = ""
        if not currencies:
            currencies = []
        if not categories:
            categories = []

        self.name = name
        self.company = company
        self.url = url
        self.platform = platform
        self.version = version
        self.agency = agency
        self.email = email
        self.currencies = currencies
        self.categories = categories
        self.delivery_options = delivery_options
        self.pickup_options = pickup_options
        self.enable_auto_discounts = enable_auto_discounts
        self.offers = offers
        self.gifts = gifts
        self.promos = promos

    @property
    def url(self):
        return self._url

    @url.setter
    def url(self, value: str):
        if len(value) > 512:
            raise ValidationError("The maximum url length is 512 characters.")
        self._url = value

    def create_dict(self, **kwargs) -> dict:
        return dict(
            name=self.name,
            company=self.company,
            url=self.url,
            platform=self.platform,
            version=self.version,
            agency=self.agency,
            email=self.email,
            currencies=[c.to_dict() for c in self.currencies],
            categories=[c.to_dict() for c in self.categories],
            delivery_options=[o.to_dict() for o in self.delivery_options],
            pickup_options=[o.to_dict() for o in self.pickup_options],
            enable_auto_discounts=self.enable_auto_discounts,
            offers=[o.to_dict() for o in self.offers],
            gifts=[g.to_dict() for g in self.gifts] if self.gifts else [],
            promos=[p.to_dict() for p in self.promos] if self.promos else [],
        )

    def create_xml(self, **kwargs) -> XMLElement:
        shop_el = XMLElement("shop")

        # Add simple elements
        for tag in (
            "name",
            "company",
            "url",
            "platform",
            "version",
            "agency",
            "email",
        ):
            value = getattr(self, tag)
            if value:
                el = XMLSubElement(shop_el, tag)
                el.text = value

        # Add currencies
        currencies_el = XMLSubElement(shop_el, "currencies")
        for c in self.currencies:
            c.to_xml(currencies_el)

        # Add categories
        categories_el = XMLSubElement(shop_el, "categories")
        for c in self.categories:
            c.to_xml(categories_el)

        # Add delivery options
        if self.delivery_options:
            delivery_options_el = XMLSubElement(shop_el, "delivery-options")
            for o in self.delivery_options:
                o.to_xml(delivery_options_el)

        # Add pickup options
        if self.pickup_options:
            pickup_options_el = XMLSubElement(shop_el, "pickup-options")
            for o in self.pickup_options:
                o.to_xml(pickup_options_el)

        # Add enable_auto_discounts
        if self._enable_auto_discounts:
            enable_auto_discounts_el = XMLSubElement(shop_el, "enable_auto_discounts")
            enable_auto_discounts_el.text = self._enable_auto_discounts

        # Add offers
        offers_el = XMLSubElement(shop_el, "offers")
        for o in self.offers:
            o.to_xml(offers_el)

        # Add gifts
        if self.gifts:
            gifts_el = XMLSubElement(shop_el, "gifts")
            for g in self.gifts:
                g.to_xml(gifts_el)

        # Add promos
        if self.promos:
            promos_el = XMLSubElement(shop_el, "promos")
            for p in self.promos:
                p.to_xml(promos_el)

        return shop_el

    @staticmethod
    def from_xml(shop_el: XMLElement) -> "Shop":
        kwargs = {}

        for el in shop_el:
            if el.tag == "currencies":
                currencies = []
                for currency_el in el:
                    currencies.append(models.Currency.from_xml(currency_el))
                kwargs["currencies"] = currencies
            elif el.tag == "categories":
                categories = []
                for category_el in el:
                    categories.append(models.Category.from_xml(category_el))
                kwargs["categories"] = categories
            elif el.tag == "delivery-options":
                delivery_options = []
                for option_el in el:
                    delivery_options.append(models.Option.from_xml(option_el))
                kwargs["delivery_options"] = delivery_options
            elif el.tag == "pickup-options":
                pickup_options = []
                for option_el in el:
                    pickup_options.append(models.Option.from_xml(option_el))
                kwargs["pickup_options"] = pickup_options
            elif el.tag == "offers":
                offers = []
                for offer_el in el:
                    offer_type = offer_el.attrib.get("type")
                    if offer_type is None:
                        offer = models.SimplifiedOffer.from_xml(offer_el)
                    elif offer_type == "vendor.model":
                        offer = models.ArbitraryOffer.from_xml(offer_el)
                    elif offer_type == "book":
                        offer = models.BookOffer.from_xml(offer_el)
                    elif offer_type == "audiobook":
                        offer = models.AudioBookOffer.from_xml(offer_el)
                    elif offer_type == "artist.title":
                        offer = models.MusicVideoOffer.from_xml(offer_el)
                    elif offer_type == "medicine":
                        offer = models.MedicineOffer.from_xml(offer_el)
                    elif offer_type == "event-ticket":
                        offer = models.EventTicketOffer.from_xml(offer_el)
                    elif offer_type == "alco":
                        offer = models.AlcoholOffer.from_xml(offer_el)
                    else:
                        raise exceptions.ParseError("Got unexpected offer type: {0}".format(offer_type))
                    offers.append(offer)
                kwargs["offers"] = offers
            elif el.tag == "gifts":
                gifts = []
                for gift_el in el:
                    gifts.append(models.Gift.from_xml(gift_el))
                if gifts:
                    kwargs["gifts"] = gifts
            elif el.tag == "promos":
                promos = []
                for promo_el in el:
                    promos.append(models.Promo.from_xml(promo_el))
                if promos:
                    kwargs["promos"] = promos
            else:
                kwargs[el.tag] = el.text

        return Shop(**kwargs)



from zs_utils.api.ebay.base_api import EbayTradingAPI


__all__ = [
    "UploadImageToEbayByURL",
]


class UploadImageToEbayByURL(EbayTradingAPI):
    """
    Загрузка на хостинг eBay изображения по его ссылке.
    Документация: https://developer.ebay.com/devzone/xml/docs/reference/ebay/UploadSiteHostedPictures.html
    """

    method_name = "UploadSiteHostedPictures"

    def get_params(self, picture_url: str, days_to_expire: int = 5, replace_all: bool = False, **kwargs):
        assert 0 < days_to_expire < 30

        if replace_all:
            policy = "ClearAndAdd"
        else:
            policy = "Add"

        return {
            "ExternalPictureURL": picture_url,
            "ExtensionInDays": days_to_expire,
            "PictureUploadPolicy": policy,
        }



from zs_utils.api.ebay.base_api import EbayTradingAPI


__all__ = [
    "GetStoreInfo",
]


class GetStoreInfo(EbayTradingAPI):
    """
    Docs:
    https://developer.ebay.com/devzone/xml/docs/reference/ebay/GetStore.html
    """

    method_name = "GetStore"

    def get_params(self, category_structure_only: bool = False, **kwargs):
        return {
            "CategoryStructureOnly": category_structure_only,
        }



from zs_utils.api.ebay.base_api import EbayAPI


class CreateOrReplaceInventoryItemGroup(EbayAPI):
    """
    Docs:
    https://developer.ebay.com/api-docs/sell/inventory/resources/inventory_item_group/methods/createOrReplaceInventoryItemGroup
    """  # noqa

    http_method = "PUT"
    resource_method = "sell/inventory/v1/inventory_item_group/{inventoryItemGroupKey}"
    required_params = ["inventoryItemGroupKey"]


class DeleteInventoryItemGroup(EbayAPI):
    """
    Docs:
    https://developer.ebay.com/api-docs/sell/inventory/resources/inventory_item_group/methods/deleteInventoryItemGroup
    """

    http_method = "DELETE"
    resource_method = "sell/inventory/v1/inventory_item_group/{inventoryItemGroupKey}"
    required_params = ["inventoryItemGroupKey"]


class GetInventoryItemGroup(EbayAPI):
    """
    Docs:
    https://developer.ebay.com/api-docs/sell/inventory/resources/inventory_item_group/methods/getInventoryItemGroup
    """

    http_method = "GET"
    resource_method = "sell/inventory/v1/inventory_item_group/{inventoryItemGroupKey}"
    required_params = ["inventoryItemGroupKey"]


class PublishOfferByInventoryItemGroup(EbayAPI):
    """
    Docs:
    https://developer.ebay.com/api-docs/sell/inventory/resources/offer/methods/publishOfferByInventoryItemGroup
    """

    http_method = "POST"
    resource_method = "sell/inventory/v1/offer/publish_by_inventory_item_group"


class WithdrawOfferByInventoryItemGroup(EbayAPI):
    """
    Docs:
    https://developer.ebay.com/api-docs/sell/inventory/resources/offer/methods/withdrawOfferByInventoryItemGroup
    """

    http_method = "POST"
    resource_method = "sell/inventory/v1/offer/withdraw_by_inventory_item_group"



from model_utils import Choices


MessageStatusEnum = Choices(
    ("Answered", "Answered"),
    ("Unanswered", "Unanswered"),
)

MessageTypeEnum = Choices(
    ("All", "All"),
    ("AskSellerQuestion", "AskSellerQuestion"),
    ("ClassifiedsBestOffer", "ClassifiedsBestOffer"),
    ("ClassifiedsContactSeller", "ClassifiedsContactSeller"),
    ("ContactEbayMember", "ContactEbayMember"),
    ("ContacteBayMemberViaAnonymousEmail", "ContacteBayMemberViaAnonymousEmail"),
    ("ContacteBayMemberViaCommunityLink", "ContacteBayMemberViaCommunityLink"),
    ("ContactMyBidder", "ContactMyBidder"),
    ("ContactTransactionPartner", "ContactTransactionPartner"),
    ("ResponseToASQQuestion", "ResponseToASQQuestion"),
    ("ResponseToContacteBayMember", "ResponseToContacteBayMember"),
)

QuestionTypeEnum = Choices(
    ("CustomCode", "CustomCode"),
    ("CustomizedSubject", "CustomizedSubject"),
    ("General", "General"),
    ("MultipleItemShipping", "MultipleItemShipping"),
    ("None", "None"),
    ("Payment", "Payment"),
    ("Shipping", "Shipping"),
)



from zs_utils.api.fedex.base_api import FedexAPI


class FedexGetRateShipment(FedexAPI):
    http_method = "POST"
    resource_method = "/rate/v1/rates/quotes"
    required_params = [
        "accountNumber",
        "requestedShipment",
    ]
    allowed_params = [
        "rateRequestControlParameters",
        "carrierCodes",
    ]



from zs_utils.api.ebay.base_api import EbayAPI


class GetProduct(EbayAPI):
    """
    Docs:
    https://developer.ebay.com/api-docs/commerce/catalog/resources/product/methods/getProduct
    """

    http_method = "GET"
    resource_method = "commerce/catalog/v1_beta/product/{epid}"
    required_params = ["epid"]


class SearchProduct(EbayAPI):
    """
    Docs:
    https://developer.ebay.com/api-docs/commerce/catalog/resources/product_summary/methods/search
    """

    http_method = "GET"
    resource_method = "commerce/catalog/v1_beta/product_summary/search"
    allowed_params = [
        "q",
        "gtin",
        "mpn",
        "category_ids",
        "limit",
        "offset",
    ]



from celery import current_app as app

from zs_utils.websocket import services


@app.task()
def send_user_message_to_websocket_task(user_id: int, event_type: str, data: dict, **kwargs) -> None:
    services.WebsocketService.send_data_to_user(
        user_id=user_id,
        content={
            "event_type": event_type,
            "data": data,
        },
    )


@app.task()
def send_notification_group_message_to_websocket_task(
    user_id: int,
    notification: str,
    data: dict,
    **kwargs,
) -> None:
    services.WebsocketService.send_data_to_notification_group(
        user_id=user_id,
        notification=notification,
        data=data,
    )



from zs_utils.api.aliexpress.base_api import AliexpressAPI


class GetCategoryTreeAPI(AliexpressAPI):
    resource_method = "aliexpress.solution.seller.category.tree.query"
    required_params = ["category_id", "filter_no_permission"]


class GetCategoryAttributesAPI(AliexpressAPI):
    resource_method = "aliexpress.category.redefining.getallchildattributesresult"
    required_params = ["cate_id"]
    allowed_params = ["parent_attrvalue_list", "locale"]


class GetCategoryInfoAPI(AliexpressAPI):
    resource_method = "aliexpress.category.redefining.getpostcategorybyid"
    required_params = ["param0"]



from zs_utils.api.ozon.base_api import OzonAPI


class OzonGetChatListAPI(OzonAPI):
    """
    Docs: https://docs.ozon.ru/api/seller/#operation/ChatAPI_ChatListV2
    """

    resource_method = "v2/chat/list"
    allowed_params = [
        "filters",
        "limit",
        "offset",
    ]


class OzonGetMessageListAPI(OzonAPI):
    """
    Docs: https://docs.ozon.ru/api/seller/#operation/ChatAPI_ChatHistoryV2
    """

    resource_method = "v2/chat/history"
    required_params = [
        "chat_id",
    ]
    allowed_params = [
        "direction",
        "from_message_id",
        "limit",
    ]


class OzonSendFileAPI(OzonAPI):
    """
    Docs: https://docs.ozon.ru/api/seller/#operation/ChatAPI_ChatSendFile
    """

    resource_method = "v1/chat/send/file"
    required_params = ["base64_content", "chat_id", "name"]


class OzonSendMessageAPI(OzonAPI):
    """
    Docs: https://docs.ozon.ru/api/seller/#operation/ChatAPI_ChatSendMessage
    """

    resource_method = "v1/chat/send/message"
    required_params = [
        "chat_id",
        "text",
    ]


class OzonCreateChatAPI(OzonAPI):
    """
    Docs: https://docs.ozon.ru/api/seller/#operation/ChatAPI_ChatStart
    """

    resource_method = "v1/chat/start"
    required_params = [
        "posting_number",
    ]


class OzonMarkAsReadAPI(OzonAPI):
    """
    Docs: https://docs.ozon.ru/api/seller/#operation/ChatAPI_ChatReadV2
    """

    resource_method = "v2/chat/read"
    required_params = [
        "chat_id",
        "from_message_id",
    ]



from zs_utils.api.ebay.base_api import EbayAPI


class TaxonomyAPI(EbayAPI):
    @property
    def headers(self):
        headers = {
            "Authorization": f"Bearer {self.access_token}",
            "Accept": "application/json",
            "Content-Type": "application/json",
            "Accept-Encoding": "application/gzip",
        }
        return headers


class GetDefaultCategoryTreeId(TaxonomyAPI):
    """
    Docs:
    https://developer.ebay.com/api-docs/commerce/taxonomy/resources/category_tree/methods/getDefaultCategoryTreeId
    """

    http_method = "GET"
    resource_method = "commerce/taxonomy/v1/get_default_category_tree_id"
    required_params = ["marketplace_id"]


class GetCategoryTree(TaxonomyAPI):
    """
    Docs:
    https://developer.ebay.com/api-docs/commerce/taxonomy/resources/category_tree/methods/getCategoryTree
    """

    http_method = "GET"
    resource_method = "commerce/taxonomy/v1/category_tree/{category_tree_id}"
    required_params = ["category_tree_id"]


class GetCategorySubtree(TaxonomyAPI):
    """
    Docs:
    https://developer.ebay.com/api-docs/commerce/taxonomy/resources/category_tree/methods/getCategorySubtree
    """

    http_method = "GET"
    resource_method = "commerce/taxonomy/v1/category_tree/{category_tree_id}/get_category_subtree"
    required_params = ["category_tree_id", "category_id"]


class GetCategorySuggestions(TaxonomyAPI):
    """
    Docs:
    https://developer.ebay.com/api-docs/commerce/taxonomy/resources/category_tree/methods/getCategorySuggestions
    """

    http_method = "GET"
    resource_method = "commerce/taxonomy/v1/category_tree/{category_tree_id}/get_category_suggestions"
    required_params = ["category_tree_id", "q"]


class GetItemAspectsForCategory(TaxonomyAPI):
    """
    Docs:
    https://developer.ebay.com/api-docs/commerce/taxonomy/resources/category_tree/methods/getItemAspectsForCategory
    """

    http_method = "GET"
    resource_method = "commerce/taxonomy/v1/category_tree/{category_tree_id}/get_item_aspects_for_category"
    required_params = ["category_tree_id", "category_id"]


class GetCompatibilityProperties(TaxonomyAPI):
    """
    Docs:
    https://developer.ebay.com/api-docs/commerce/taxonomy/resources/category_tree/methods/getCompatibilityProperties
    """

    http_method = "GET"
    resource_method = "commerce/taxonomy/v1/category_tree/{category_tree_id}/get_compatibility_properties"
    required_params = ["category_tree_id", "category_id"]


class GetCompatibilityPropertyValues(TaxonomyAPI):
    """
    Docs:
    https://developer.ebay.com/api-docs/commerce/taxonomy/resources/category_tree/methods/getCompatibilityPropertyValues
    """

    http_method = "GET"
    resource_method = "commerce/taxonomy/v1/category_tree/{category_tree_id}/get_compatibility_property_values"
    required_params = ["category_tree_id", "compatibility_property", "category_id"]
    allowed_params = ["filter"]



from zs_utils.api.etsy.base_api import EtsyAPI


class GetReceipt(EtsyAPI):
    """
    Requests a Shop Receipt with a specific receipt id from a Shop.

    Docs:
    https://www.etsy.com/openapi/developers#operation/getShopReceipt
    """

    http_method = "GET"
    resource_method = "shops/{shop_id}/receipts/{receipt_id}"
    required_params = ["shop_id", "receipt_id"]


class FindAllShopReceipts(EtsyAPI):
    """
    Retrieves a set of Receipt objects associated to a Shop.

    Docs:
    https://www.etsy.com/openapi/developers#operation/getShopReceipts
    """

    http_method = "GET"
    resource_method = "shops/{shop_id}/receipts"
    required_params = ["shop_id"]
    allowed_params = [
        "min_created",
        "max_created",
        "min_last_modified",
        "max_last_modified",
        "limit",
        "offset",
        "was_paid",
        "was_shipped",
    ]


class SubmitTracking(EtsyAPI):
    """
    Submits tracking information and sends a shipping notification email to the buyer.
    If send_bcc is true, the shipping notification will be sent to the seller as well.

    Docs:
    https://www.etsy.com/openapi/developers#operation/createReceiptShipment
    """

    http_method = "POST"
    resource_method = "shops/{shop_id}/receipts/{receipt_id}/tracking"
    required_params = [
        "shop_id",
        "receipt_id",
    ]
    allowed_params = [
        "tracking_code",
        "carrier_name",
        "send_bcc",
    ]



from zs_utils.api.shopify.base_api import ShopifyAPI


class GetShopifyShopInfoAPI(ShopifyAPI):
    """
    https://shopify.dev/api/admin-rest/2022-01/resources/shop#get-shop?fields=address1,address2,city,province,country
    """

    http_method = "GET"
    resource_method = "shop.json"
    allowed_params = ["fields"]  # address1,address2,city,province,country



import inspect
from sys import modules
from importlib import import_module


__all__ = [
    "get_class_by_method",
    "get_func_path",
    "get_class_method_path",
    "get_callable_path",
    "import_object",
    "import_object_by_path",
    "import_class_attr",
    "import_class_attr_by_path",
]


def get_class_by_method(method):
    """
    Получения класса по его методу
    """

    return vars(modules[method.__module__])[method.__qualname__.split(".")[0]]


def get_func_path(func) -> str:
    """
    Получение пути функции.
    """

    return ".".join([func.__module__, func.__name__])


def get_class_method_path(method) -> str:
    """
    Получение пути метода класса.
    """

    return ".".join([method.__module__, get_class_by_method(method).__name__, method.__name__])


def get_callable_path(obj) -> str:
    if inspect.isclass(obj):
        return get_func_path(func=obj)
    elif inspect.isfunction(obj):
        if "." in obj.__qualname__:  # Статический метод класса
            return get_class_method_path(method=obj)
        else:
            return get_func_path(func=obj)
    elif inspect.ismethod(obj):
        return get_class_method_path(method=obj)
    else:
        raise ValueError(f"Неизвестный тип объекта: '{type(obj)}'")


def import_object(module_name: str, object_name: str):
    # Получение модуля
    module = import_module(module_name)

    # Получение функции модуля
    func = getattr(module, object_name, None)
    if not func:
        raise AttributeError(f"Объект '{object_name}' не найден в модуле '{module_name}'.")

    return func


def import_object_by_path(path: str):
    path_parts = path.split(".")
    return import_object(module_name=".".join(path_parts[:-1]), object_name=path_parts[-1])


def import_class_attr(module_name: str, class_name: str, attr_name: str):
    # Получение модуля
    module = import_module(module_name)

    # Получение класса
    class_obj = getattr(module, class_name, None)
    if not class_obj:
        raise AttributeError(f"Класс '{class_name}' не найден в модуле '{module_name}'.")

    # Получение метода класса
    try:
        class_method = getattr(class_obj, attr_name)
    except AttributeError:
        raise AttributeError(f"Метод '{attr_name}' не найден у класса '{class_name}'.")

    return class_method


def import_class_attr_by_path(path: str):
    path_parts = path.split(".")
    return import_class_attr(
        module_name=".".join(path_parts[:-2]),
        class_name=path_parts[-2],
        attr_name=path_parts[-1],
    )



import datetime

from zs_utils.api.ebay.base_api import EbayTradingAPI


__all__ = [
    "GetMessagesInfo",
    "GetMessageSummary",
    "GetMessageHeaderList",
    "GetMessageList",
    "SendMessage",
    "GetMemberMessageList",
    "AnswerOrderMessage",
    "MarkMessageRead",
    "MarkMessageUnread",
    "DeleteMessageList",
]


class GetMessagesInfo(EbayTradingAPI):
    """
    GetMyMessages:
    https://developer.ebay.com/Devzone/XML/docs/Reference/eBay/GetMyMessages.html
    """

    method_name = "GetMyMessages"

    def get_params(
        self,
        detail_level: str,
        folder_id: int = None,
        message_ids: list = [],
        days_ago: int = None,
        **kwargs,
    ):
        assert detail_level in ["ReturnHeaders", "ReturnMessages", "ReturnSummary"]
        assert folder_id in [None, "0", "1", "2", 0, 1, 2]

        if message_ids:
            assert len(message_ids) <= 10, 'Размер "message_ids" не должен превышать 10.'
            folder_id = None
            days_ago = None
        elif detail_level == "ReturnMessages":
            raise AttributeError('Необходимо задать "message_ids".')

        if days_ago:
            EndTime = datetime.datetime.now() + datetime.timedelta(minutes=1)
            StartTime = EndTime - datetime.timedelta(days=int(days_ago))
        else:
            StartTime = None
            EndTime = None

        return {
            "DetailLevel": detail_level,
            "FolderID": folder_id,
            "MessageIDs": {"MessageID": message_ids},
            "EndTime": EndTime,
            "StartTime": StartTime,
        }


class GetMessageSummary(GetMessagesInfo):
    def get_params(self, **kwargs):
        kwargs["detail_level"] = "ReturnSummary"
        kwargs["folder_id"] = None
        kwargs["message_ids"] = None
        return super().get_params(**kwargs)


class GetMessageHeaderList(GetMessagesInfo):
    def get_params(self, full_messages=False, **kwargs):
        kwargs["detail_level"] = "ReturnHeaders"
        return super().get_params(**kwargs)


class GetMessageList(GetMessagesInfo):
    def get_params(self, **kwargs):
        kwargs["detail_level"] = "ReturnMessages"
        return super().get_params(**kwargs)


class AbstractSendMessage(EbayTradingAPI):
    """
    Abstract class.
    """

    def get_params(
        self,
        message_body: str,
        parent_message_id: str,
        message_media_url: str = None,
        item_id: str = None,
        recipient_id: str = None,
        email_copy_to_sender: bool = False,
        **kwargs,
    ):
        params = {
            "MemberMessage": {
                "Body": message_body,
                "EmailCopyToSender": email_copy_to_sender,
                "ParentMessageID": parent_message_id,
                "RecipientID": recipient_id,
            },
            "ItemID": item_id,
        }

        if message_media_url:
            params["MemberMessage"]["MessageMedia"] = {
                "MediaName": "Attached media",
                "MediaURL": message_media_url,
            }

        return params


class SendMessage(AbstractSendMessage):
    """
    AddMemberMessageRTQ:
    https://developer.ebay.com/Devzone/XML/docs/Reference/eBay/AddMemberMessageRTQ.html
    """

    method_name = "AddMemberMessageRTQ"

    def get_params(self, display_to_public=False, **kwargs):
        if not kwargs.get("item_id", None):
            display_to_public = False
            if not kwargs.get("recipient_id", None):
                raise AttributeError('Необходимо задать "recipient_id" или "item_id"')

        params = super().get_params(**kwargs)

        params["MemberMessage"].update({"DisplayToPublic": display_to_public})
        return params


class GetMemberMessageList(EbayTradingAPI):
    """
    GetMemberMessages:
    https://developer.ebay.com/Devzone/XML/docs/Reference/eBay/GetMemberMessages.html
    """

    method_name = "GetMemberMessages"

    def get_params(
        self,
        message_type="All",
        message_status=None,
        item_id=None,
        sender_id=None,
        days_ago: int = None,
        **kwargs,
    ):
        if message_type not in ["All", "AskSellerQuestion"]:
            raise AttributeError('Недопустимое значение параметра "message_type"')

        params = {"MailMessageType": message_type}

        if message_status:
            if not (message_status in ["Answered", "Unanswered"]):
                raise AttributeError('Недопустимое значение параметра "message_status"')
            params.update({"MessageStatus": message_status})

        if item_id:
            params.update({"ItemID": item_id})
        elif sender_id:
            params.update({"SenderID": sender_id})
        else:
            end_creation_time = datetime.datetime.now()
            start_creation_time = end_creation_time - datetime.timedelta(days=days_ago)

            params.update(
                {
                    "EndCreationTime": end_creation_time,
                    "StartCreationTime": start_creation_time,
                }
            )

        return params


class AnswerOrderMessage(AbstractSendMessage):
    """
    AddMemberMessageAAQToPartner:
    https://developer.ebay.com/Devzone/XML/docs/Reference/eBay/AddMemberMessageAAQToPartner.html
    HINT: item needs to be a part of an offer
    """

    method_name = "AddMemberMessageAAQToPartner"

    question_type_enum = [
        "CustomizedSubject",
        "General",
        "MultipleItemShipping",
        "None",
        "Payment",
        "Shipping",
    ]

    def get_params(self, item_id, recipient_id, subject, question_type="None", **kwargs):
        if not (question_type in self.question_type_enum):
            raise AttributeError('Недопустимое значение параметра "question_type"')

        params = super().get_params(**kwargs)
        params["MemberMessage"].update(
            {
                "QuestionType": question_type,
                "Subject": subject,
            }
        )
        return params


class ReviseMessages(EbayTradingAPI):
    """
    ReviseMyMessages:
    https://developer.ebay.com/Devzone/XML/docs/Reference/eBay/ReviseMyMessages.html
    """

    method_name = "ReviseMyMessages"

    def get_params(self, message_ids: list, read: bool = None, folder_id: int = None, **kwargs):
        if read:
            folder_id = None

        return {
            "MessageIDs": {
                "MessageID": message_ids,
            },
            "Read": read,
            "FolderID": folder_id,
        }


class MarkMessageRead(ReviseMessages):
    def get_params(self, **kwargs):
        kwargs["read"] = True
        return super().get_params(**kwargs)


class MarkMessageUnread(ReviseMessages):
    def get_params(self, **kwargs):
        kwargs["read"] = False
        return super().get_params(**kwargs)


class DeleteMessageList(EbayTradingAPI):
    """
    Docs:
    https://developer.ebay.com/Devzone/XML/docs/Reference/eBay/DeleteMyMessages.html
    """

    method_name = "DeleteMyMessages"

    def get_params(self, message_ids: list, **kwargs):
        return {
            "MessageIDs": {"MessageID": message_ids},
        }



from zs_utils.api.ozon.base_api import OzonAPI


class OzonGetWarehouseListAPI(OzonAPI):
    resource_method = "v1/warehouse/list"


class OzonGetWarehouseMethodListAPI(OzonAPI):
    resource_method = "v1/delivery-method/list"
    required_params = ["limit"]
    allowed_params = ["filter", "offset"]



from zs_utils.api.ebay.base_api import EbayAPI


# SINGLE ITEM API


class CreateOrReplaceInventoryItem(EbayAPI):
    """
    Docs:
    https://developer.ebay.com/api-docs/sell/inventory/resources/inventory_item/methods/createOrReplaceInventoryItem
    """

    http_method = "PUT"
    resource_method = "sell/inventory/v1/inventory_item/{sku}"
    required_params = ["sku"]


class GetInventoryItem(EbayAPI):
    """
    Docs:
    https://developer.ebay.com/api-docs/sell/inventory/resources/inventory_item/methods/getInventoryItem
    """

    http_method = "GET"
    resource_method = "sell/inventory/v1/inventory_item/{sku}"
    required_params = ["sku"]


class GetInventoryItems(EbayAPI):
    """
    Docs:
    https://developer.ebay.com/api-docs/sell/inventory/resources/inventory_item/methods/getInventoryItems
    """

    http_method = "GET"
    resource_method = "sell/inventory/v1/inventory_item"
    allowed_params = ["offset", "limit"]


class DeleteInventoryItem(EbayAPI):
    """
    Docs:
    https://developer.ebay.com/api-docs/sell/inventory/resources/inventory_item/methods/deleteInventoryItem
    """

    http_method = "DELETE"
    resource_method = "sell/inventory/v1/inventory_item/{sku}"
    required_params = ["sku"]


# PRODUCT COMPATIBILITY API


class ProductCompatibilityAPI(EbayAPI):
    resource_method = "sell/inventory/v1/inventory_item/{sku}/product_compatibility"
    required_params = ["sku"]


class CreateOrReplaceProductCompatibility(ProductCompatibilityAPI):
    """
    Docs:
    https://developer.ebay.com/api-docs/sell/inventory/resources/inventory_item/product_compatibility/methods/createOrReplaceProductCompatibility
    """  # noqa

    http_method = "PUT"


class GetProductCompatibility(ProductCompatibilityAPI):
    """
    Docs:
    https://developer.ebay.com/api-docs/sell/inventory/resources/inventory_item/product_compatibility/methods/getProductCompatibility
    """  # noqa

    http_method = "GET"


class DeleteProductCompatibility(ProductCompatibilityAPI):
    """
    Docs:
    https://developer.ebay.com/api-docs/sell/inventory/resources/inventory_item/product_compatibility/methods/deleteProductCompatibility
    """  # noqa

    http_method = "DELETE"


# BULK API


class BulkUpdatePriceQuantity(EbayAPI):
    """
    Docs:
    https://developer.ebay.com/api-docs/sell/inventory/resources/inventory_item/methods/bulkUpdatePriceQuantity
    """

    http_method = "POST"
    resource_method = "sell/inventory/v1/bulk_update_price_quantity"


class BulkCreateOrReplaceInventoryItem(EbayAPI):
    """
    Docs:
    https://developer.ebay.com/api-docs/sell/inventory/resources/inventory_item/methods/bulkCreateOrReplaceInventoryItem
    """  # noqa

    http_method = "POST"
    resource_method = "sell/inventory/v1/bulk_create_or_replace_inventory_item"


class BulkGetInventoryItem(EbayAPI):
    """
    Docs:
    https://developer.ebay.com/api-docs/sell/inventory/resources/inventory_item/methods/bulkGetInventoryItem
    """

    http_method = "POST"
    resource_method = "sell/inventory/v1/bulk_get_inventory_item"



from model_utils.models import TimeStampedModel, UUIDModel
from rest_framework import status

from django.db import models
from django.conf import settings

from zs_utils.json_utils import CustomJSONEncoder
from zs_utils.api import constants


__all__ = [
    "AbstractAPIRequestLog",
    "APIRequestLog",
]


class AbstractAPIRequestLog(TimeStampedModel, UUIDModel):
    """
    Хранение запросов к внешним сервисам
    """

    user = models.ForeignKey(
        to=settings.AUTH_USER_MODEL,
        null=True,
        on_delete=models.CASCADE,
        related_name="api_request_logs",
        related_query_name="api_request_log",
        verbose_name="Пользователь",
    )

    # Данные запроса
    url = models.TextField(verbose_name="URL запроса")
    method = models.CharField(choices=constants.HTTP_METHODS, max_length=6, verbose_name="Метод запроса")
    params = models.JSONField(
        blank=True,
        null=True,
        encoder=CustomJSONEncoder,
        verbose_name="Параметры URL",
    )
    request_headers = models.JSONField(
        blank=True,
        null=True,
        encoder=CustomJSONEncoder,
        verbose_name="Хедеры запроса",
    )
    request_body = models.JSONField(
        blank=True,
        null=True,
        encoder=CustomJSONEncoder,
        verbose_name="Тело запроса",
    )

    # Данные ответа
    response_time = models.IntegerField(verbose_name="Время ответа (миллисекунда)")
    status_code = models.IntegerField(verbose_name="Код ответа")
    response_headers = models.JSONField(
        blank=True,
        null=True,
        encoder=CustomJSONEncoder,
        verbose_name="Хедеры ответа",
    )
    response_body = models.JSONField(
        blank=True,
        null=True,
        encoder=CustomJSONEncoder,
        verbose_name="Тело ответа",
    )

    @property
    def is_success(self) -> bool:
        return status.is_success(code=self.status_code)

    class Meta:
        abstract = True


class APIRequestLog(AbstractAPIRequestLog):
    class Meta:
        ordering = ["-created"]



from django.conf import settings

from zs_utils.api.base_api import BaseAPI


class ShopifyAPI(BaseAPI):
    payload_key = None

    def __init__(self, access_token: str, shop_url: str, **kwargs):
        self.access_token = access_token
        self.shop_url = shop_url

        super().__init__(**kwargs)

    @property
    def production_api_url(self):
        return f"https://{self.shop_url}/admin/api/{settings.SHOPIFY_API_VERSION}/"

    @property
    def headers(self) -> dict:
        return {
            "Content-Type": "application/json",
            "X-Shopify-Access-Token": self.access_token,
        }

    def get_payload(self, params: dict):
        return {self.payload_key: params} if self.payload_key else params



import jwt
from urllib.parse import parse_qs
from channels.middleware import BaseMiddleware
from channels.db import database_sync_to_async
from rest_framework_simplejwt.exceptions import InvalidToken, TokenError

from django.conf import settings
from django.contrib.auth import get_user_model


class JWTAuthMiddleware(BaseMiddleware):
    @database_sync_to_async
    def get_user(self, user_id: int):
        return get_user_model().objects.get(id=user_id)

    async def __call__(self, scope, receive, send):
        from rest_framework_simplejwt.tokens import UntypedToken

        if scope["query_string"]:
            query_data: dict = parse_qs(scope["query_string"].decode("utf-8"))
        else:
            query_data = {}

        # ID подключения
        scope["connection_id"] = query_data.get("connection_id")

        # Получение токена

        if query_data.get("token"):
            token = query_data["token"][0]
        else:
            token = None

        if not token:
            for header in scope["headers"]:
                key = header[0].decode("utf-8")
                if key.lower() == "authorization":
                    token = header[1].decode("utf-8")
                    break

        scope["token"] = token

        # Получение пользователя
        if "user" not in scope:
            user = None

            if token:
                try:
                    UntypedToken(token)
                except (InvalidToken, TokenError):
                    # TODO: флаг устаревшего токена
                    pass
                else:
                    decoded_data = jwt.decode(jwt=token, key=settings.SECRET_KEY, algorithms=["HS256"])
                    user = await self.get_user(user_id=decoded_data["user_id"])

            scope["user"] = user

        return await super().__call__(scope, receive, send)



from zs_utils.api.ebay.base_api import EbayAPI


class GetListingStructurePolicies(EbayAPI):
    """
    Docs:
    https://developer.ebay.com/api-docs/sell/metadata/resources/marketplace/methods/getListingStructurePolicies
    """

    http_method = "GET"
    resource_method = "sell/metadata/v1/marketplace/{marketplace_id}/get_listing_structure_policies"
    required_params = ["marketplace_id"]
    allowed_params = ["filter"]


class GetItemConditionPolicies(EbayAPI):
    """
    Docs:
    https://developer.ebay.com/api-docs/sell/metadata/resources/marketplace/methods/getItemConditionPolicies
    """

    http_method = "GET"
    resource_method = "sell/metadata/v1/marketplace/{marketplace_id}/get_item_condition_policies"
    required_params = ["marketplace_id"]
    allowed_params = ["filter"]


class GetAutomotivePartsCompatibilityPolicies(EbayAPI):
    """
    Docs:
    https://developer.ebay.com/api-docs/sell/metadata/resources/marketplace/methods/getAutomotivePartsCompatibilityPolicies
    """  # noqa

    http_method = "GET"
    resource_method = "sell/metadata/v1/marketplace/{marketplace_id}/get_automotive_parts_compatibility_policies"
    required_params = ["marketplace_id"]
    allowed_params = ["filter"]



from channels.generic.websocket import AsyncJsonWebsocketConsumer
from channels.db import database_sync_to_async

from django.conf import settings
from django.contrib.auth import get_user_model

from zs_utils.json_utils import custom_json_dumps, custom_json_loads


class BaseChannelsConsumer(AsyncJsonWebsocketConsumer):
    @database_sync_to_async
    def get_user(self, user_id: int):
        return get_user_model().objects.get(id=user_id)

    async def connect(self):
        if self.scope["user"]:
            self.user = self.scope["user"]
        else:
            if self.scope["token"]:
                # Соединение отвергается, если был передан невалидный токен
                if getattr(settings, "REQUIRE_JWT_FOR_WEBSOCKET", True):
                    await self.close(code=403)
                    return None
                # В режиме отладки используется тестовый пользователь
                self.user = await self.get_user(user_id=1)
            else:
                self.user = None

        if self.user:
            # Для каждого пользователя создается своя группа
            await self.add_to_group(self.get_user_group_name(user_id=self.user.id))
        elif self.scope.get("connection_id"):
            # Создание группы для анонимного подключения
            await self.add_to_group(self.get_anon_group_name(connection_id=self.scope["connection_id"]))

        self.pretenders = {}

        await self.accept()

    async def disconnect(self, code):
        for group_name in self.groups:
            await self.remove_from_group(group_name=group_name)
        await self.close(code=code)

    async def add_to_group(self, group_name: str):
        await self.channel_layer.group_add(group_name, self.channel_name)
        if group_name not in self.groups:
            self.groups.append(group_name)

    async def remove_from_group(self, group_name: str):
        await self.channel_layer.group_discard(group_name, self.channel_name)
        if group_name in self.groups:
            self.groups.remove(group_name)

    @staticmethod
    def get_user_group_name(user_id: int):
        return f"user_group_{user_id}"

    @staticmethod
    def get_anon_group_name(connection_id: str):
        return f"anon_group_{connection_id}"

    @staticmethod
    def get_user_notification_group_name(user_id: int, notification: str):
        return f"notification_{notification}_group_{user_id}"

    @classmethod
    async def encode_json(cls, content: dict):
        return custom_json_dumps(content, ensure_ascii=False)

    @classmethod
    async def decode_json(cls, text_data: str):
        return custom_json_loads(text_data)

    async def send_message(self, event: dict):
        await self.send(text_data=event["content"])

    async def send_error_message(self, message: str = None, errors: dict = None):
        await self.send_json(content={"is_success": False, "message": message, "errors": errors})

    async def send_success_message(self, message: str = None):
        await self.send_json(content={"is_success": True, "message": message})

    async def ping_action_processor(self, content: dict, **kwargs):
        await self.send_success_message(message="pong")

    async def disconnect_action_processor(self, content: dict, **kwargs):
        await self.disconnect(None)

    def subscribe_action_validator(self, content: dict, **kwargs):
        errors = {}

        for key in ["notification"]:
            if not content.get(key):
                errors[key] = "Обязательное поле."

        return errors

    async def subscribe_action_processor(self, content: dict, **kwargs):
        if self.user:
            if self.user.id in self.pretenders:
                user_id = self.pretenders[self.user.id]
            else:
                user_id = self.user.id
            group_name = self.get_user_notification_group_name(
                user_id=user_id,
                notification=content["notification"],
            )
        else:
            group_name = content["notification"]

        await self.add_to_group(group_name=group_name)
        await self.send_success_message(message="Подписка создана.")

    def unsubscribe_action_validator(self, content: dict, **kwargs):
        return self.subscribe_action_validator(content=content, **kwargs)

    async def unsubscribe_action_processor(self, content: dict, **kwargs):
        if self.user:
            if self.user.id in self.pretenders:
                user_id = self.pretenders[self.user.id]
            else:
                user_id = self.user.id
            group_name = self.get_user_notification_group_name(
                user_id=user_id,
                notification=content["notification"],
            )
        else:
            group_name = content["notification"]

        await self.remove_from_group(group_name=group_name)
        await self.send_success_message(message="Подписка удалена.")

    def pretend_on_action_validator(self, content: dict, **kwargs):
        errors = {}

        for key in ["user_id"]:
            if not content.get(key):
                errors[key] = "Обязательное поле."

        return errors

    async def pretend_on_action_processor(self, content: dict, **kwargs):
        if self.user and self.user.is_staff:
            await self.add_to_group(group_name=self.get_user_group_name(user_id=content["user_id"]))
            await self.remove_from_group(group_name=self.get_user_group_name(user_id=self.user.id))
            self.pretenders[self.user.id] = content["user_id"]
            await self.send_success_message(message="Режим получения сообщений другого пользователя включен.")

    def pretend_off_action_validator(self, content: dict, **kwargs):
        return self.pretend_on_action_validator(content=content, **kwargs)

    async def pretend_off_action_processor(self, content: dict, **kwargs):
        if self.user and self.user.is_staff:
            await self.add_to_group(group_name=self.get_user_group_name(user_id=self.user.id))
            await self.remove_from_group(group_name=self.get_user_group_name(user_id=content["user_id"]))
            if self.user.id in self.pretenders:
                self.pretenders.pop(self.user.id)
            await self.send_success_message(message="Режим получения сообщений другого пользователя отключен.")

    async def receive_json(self, content: dict, **kwargs):
        action: str = content.get("action")
        if action and getattr(settings, "WEBSOCKET_ACTIONS", None):
            if action not in settings.WEBSOCKET_ACTIONS:
                return await self.send_error_message(message="Недопустимое действие.")

            validator = getattr(self, f"{action}_action_validator", None)
            if validator:
                errors: dict = validator(content=content)
                if errors:
                    return await self.send_error_message(message="Ошибка валидации", errors=errors)

            processor = getattr(self, f"{action}_action_processor")
            if processor:
                await processor(content=content, **kwargs)



from django.utils import timezone
from django.conf import settings

from zs_utils.api.amocrm.base_action import AmocrmAction
from zs_utils.api.amocrm import core


__all__ = [
    "RemoteGetAmocrmContactByLead",
    "RemoteCreateAmocrmContact",
    "RemoteCreateAmocrmLead",
    "RemoteUpdateAmocrmLead",
    "RemoteCreateAmocrmTask",
]


class RemoteGetAmocrmContactByLead(AmocrmAction):
    description = "Получение информации о сделке amoCRM"
    api_class = core.GetAmocrmLeadAPI
    required_params = ["lead_id"]
    allowed_params = ["with"]

    def get_params(self, **kwargs) -> dict:
        kwargs["with"] = "contacts"
        return super().get_params(**kwargs)

    def format_success_results(self, results: dict, **kwargs) -> dict:
        return {"contact_id": results["_embedded"]["contacts"][0]["id"]}


class RemoteCreateAmocrmContact(AmocrmAction):
    description = "Создание контакта на amoCRM"
    api_class = core.CreateAmocrmContactAPI
    SAVE_REQUEST_LOG = True
    CUSTOM_FIELDS_MAPPING = settings.AMOCRM_CONTACT_CUSTOM_FIELDS
    required_params = []
    allowed_params = [
        "name",
        "first_name",
        "last_name",
        "custom_fields_values",
        "responsible_user_id",
    ]

    def get_params(self, responsible_user_id: int = None, **kwargs) -> dict:
        if not responsible_user_id:
            responsible_user_id = settings.AMOCRM_DEFAULT_MANAGER_ID
        return super().get_params(responsible_user_id=responsible_user_id, **kwargs)

    def format_success_results(self, results: dict, **kwargs) -> dict:
        return {"contact_id": results["_embedded"]["contacts"][0]["id"]}


class RemoteCreateAmocrmLead(AmocrmAction):
    description = "Создание сделки на amoCRM"
    api_class = core.CreateAmocrmLeadAPI
    SAVE_REQUEST_LOG = True
    CUSTOM_FIELDS_MAPPING = settings.AMOCRM_LEAD_CUSTOM_FIELDS
    required_params = [
        "pipeline_id",
        "_embedded",
    ]
    allowed_params = [
        "created_by",
        "updated_by",
        "status_id",
        "custom_fields_values",
        "name",
        "price",
        "responsible_user_id",
    ]

    def get_params(
        self,
        pipeline_id: int,
        contact_id: int,
        status_id: int,
        responsible_user_id: int = None,
        **kwargs,
    ) -> dict:
        if not responsible_user_id:
            responsible_user_id = settings.AMOCRM_DEFAULT_MANAGER_ID

        kwargs.update(
            {
                "pipeline_id": pipeline_id,
                "status_id": status_id,
                "responsible_user_id": responsible_user_id,
                "_embedded": {
                    "contacts": [{"id": contact_id}],
                    "tags": [],
                },
            }
        )

        # Теги
        tags = kwargs.pop("tags", [])
        for tag in tags:
            kwargs["_embedded"]["tags"].append({"name": tag})

        return super().get_params(**kwargs)

    def format_success_results(self, results: dict, **kwargs) -> dict:
        return {"lead_id": results["_embedded"]["leads"][0]["id"]}


class RemoteUpdateAmocrmLead(AmocrmAction):
    description = "Обновление сделки amoCRM"
    api_class = core.UpdateAmocrmLeadAPI
    SAVE_REQUEST_LOG = True
    CUSTOM_FIELDS_MAPPING = settings.AMOCRM_LEAD_CUSTOM_FIELDS
    required_params = [
        "id",
    ]
    allowed_params = [
        "name",
        "price",
        "pipeline_id",
        "_embedded",
        "status_id",
        "created_by",
        "updated_by",
        "responsible_user_id",
        "loss_reason_id",
        "custom_fields_values",
    ]

    def get_params(self, lead_id: int, **kwargs) -> dict:
        kwargs["id"] = lead_id
        return super().get_params(**kwargs)


class RemoteCreateAmocrmTask(AmocrmAction):
    description = "Создание задачи на amoCRM"
    api_class = core.CreateAmocrmTaskAPI
    SAVE_REQUEST_LOG = True
    required_params = [
        "text",
        "complete_till",
    ]
    allowed_params = [
        "responsible_user_id",
        "created_by",
        "updated_by",
        "entity_id",
        "entity_type",
        "is_completed",
        "task_type_id",
        "duration",
        "result",
    ]

    def get_params(
        self,
        lead_id: str,
        responsible_user_id: int = None,
        created_user_id: int = None,
        complete_till: timezone.datetime = None,
        **kwargs,
    ) -> dict:
        if not responsible_user_id:
            responsible_user_id = settings.AMOCRM_DEFAULT_MANAGER_ID
        if not created_user_id:
            created_user_id = settings.AMOCRM_MAIN_MANAGER_ID
        kwargs.update(
            {
                "responsible_user_id": responsible_user_id,
                "created_by": created_user_id,
                "complete_till": int(complete_till.timestamp()),
                "entity_id": lead_id,
                "entity_type": "leads",
            }
        )
        return super().get_params(**kwargs)

    def format_success_results(self, results: dict, **kwargs) -> dict:
        return {"task_id": results["_embedded"]["tasks"][0]["id"]}



MarketplaceToLang = {
    "default": "en-US",
    "EBAY_US": "en-US",
    "EBAY_MOTORS_US": "en-US",
    "EBAY_CA": "en-CA",
    "EBAY_GB": "en-GB",
    "EBAY_AU": "en-AU",
    "EBAY_AT": "de-AT",
    "EBAY_BE_FR": "fr-BE",
    "EBAY_FR": "fr-FR",
    "EBAY_DE": "de-DE",
    "EBAY_IT": "it-IT",
    "EBAY_BE_NL": "nl-BE",
    "EBAY_NL": "nl-NL",
    "EBAY_ES": "es-ES",
    "EBAY_CH": "de-CH",
    "EBAY_TW": "zh-TW",
    "EBAY_CZ": "en-US",  # not defined
    "EBAY_DK": "en-US",  # not defined
    "EBAY_FI": "en-US",  # not defined
    "EBAY_GR": "en-US",  # not defined
    "EBAY_HK": "zh-HK",
    "EBAY_HU": "en-US",  # not defined
    "EBAY_IN": "en-GB",
    "EBAY_ID": "en-US",  # not defined
    "EBAY_IE": "en-IE",
    "EBAY_IL": "en-US",  # not defined
    "EBAY_MY": "en-US",
    "EBAY_NZ": "en-US",  # not defined
    "EBAY_NO": "en-US",  # not defined
    "EBAY_PH": "en-PH",
    "EBAY_PL": "pl-PL",
    "EBAY_PT": "en-US",  # not defined
    "EBAY_PR": "en-US",  # not defined
    "EBAY_RU": "ru-RU",
    "EBAY_SG": "en-US",
    "EBAY_ZA": "en-US",  # not defined
    "EBAY_SE": "en-US",  # not defined
    "EBAY_TH": "th-TH",
    "EBAY_VN": "en-US",
    "EBAY_CN": "en-US",  # not defined
    "EBAY_PE": "en-US",  # not defined
    "EBAY_CA_FR": "fr-CA",
    "EBAY_JP": "en-US",  # not defined
}

MarketplaceToLocale = {
    "default": "en_US",
    "EBAY_US": "en_US",
    "EBAY_MOTORS_US": "en_US",
    "EBAY_CA": "en_CA",
    "EBAY_GB": "en_GB",
    "EBAY_AU": "en_AU",
    "EBAY_AT": "de_AT",
    "EBAY_BE_FR": "fr_BE",
    "EBAY_FR": "fr_FR",
    "EBAY_DE": "de_DE",
    "EBAY_IT": "it_IT",
    "EBAY_BE_NL": "nl_BE",
    "EBAY_NL": "nl_NL",
    "EBAY_ES": "es_ES",
    "EBAY_CH": "de_CH",
    "EBAY_TW": "zh_TW",
    "EBAY_CZ": "en_US",  # not defined
    "EBAY_DK": "en_US",  # not defined
    "EBAY_FI": "en_US",  # not defined
    "EBAY_GR": "en_US",  # not defined
    "EBAY_HK": "zh_HK",
    "EBAY_HU": "en_US",  # not defined
    "EBAY_IN": "en_GB",
    "EBAY_ID": "en_US",  # not defined
    "EBAY_IE": "en_IE",
    "EBAY_IL": "en_US",  # not defined
    "EBAY_MY": "en_US",
    "EBAY_NZ": "en_US",  # not defined
    "EBAY_NO": "en_US",  # not defined
    "EBAY_PH": "en_PH",
    "EBAY_PL": "pl_PL",
    "EBAY_PT": "en_US",  # not defined
    "EBAY_PR": "en_US",  # not defined
    "EBAY_RU": "ru_RU",
    "EBAY_SG": "en_US",
    "EBAY_ZA": "en_US",  # not defined
    "EBAY_SE": "en_US",  # not defined
    "EBAY_TH": "th_TH",
    "EBAY_VN": "en_US",
    "EBAY_CN": "en_US",  # not defined
    "EBAY_PE": "en_US",  # not defined
    "EBAY_CA_FR": "fr_CA",
    "EBAY_JP": "en_US",  # not defined
}



from zs_utils.api.ebay.base_api import EbayAPI


class GetOrder(EbayAPI):
    """
    Docs:
    https://developer.ebay.com/api-docs/sell/fulfillment/resources/order/methods/getOrder
    """

    http_method = "GET"
    resource_method = "sell/fulfillment/v1/order/{orderId}"
    required_params = ["orderId"]


class GetOrders(EbayAPI):
    """
    Docs:
    https://developer.ebay.com/api-docs/sell/fulfillment/resources/order/methods/getOrders
    """

    http_method = "GET"
    resource_method = "sell/fulfillment/v1/order"
    allowed_params = ["orderIds", "filter", "offset", "limit"]
    MAX_OFFSET = 999
    MAX_LIMIT = 1000

    def get_clean_params(self, params: dict) -> dict:
        clean_params = super().get_clean_params(params)

        if clean_params.get("orderIds"):
            clean_params["orderIds"] = self.clean_orderIds(orderIds_string=clean_params["orderIds"])

        if clean_params.get("filter"):
            clean_params["filter"] = self.clean_filter(filter_string=clean_params["filter"])

        return clean_params

    def clean_orderIds(self, orderIds_string):
        message = ""
        order_ids = [order_id.strip() for order_id in orderIds_string.split(",")]
        if not (1 <= len(order_ids) <= 50):
            message = f"Количество ID заказов должно лежать в диапазоне [1:50]. Передано ID заказов: {len(order_ids)}"
        elif len(order_ids) != len(set(order_ids)):
            message = f"Среди ID заказов есть повторяющиеся.\nСписок ID: {orderIds_string}"
        else:
            for order_id in order_ids:
                for part in order_id.split("-"):
                    if not part.isdigit():
                        message += f"Недопустимый ID заказа: {order_id}.\n"
        if message:
            raise self.exception_class(message)
        return ",".join(order_ids)

    def clean_filter(self, filter_string):
        # Docs: https://developer.ebay.com/api-docs/sell/fulfillment/resources/order/methods/getOrders#h2-input

        def _percent_encode(string):
            string = string.replace("[", "%5B")
            string = string.replace("]", "%5D")
            string = string.replace("{", "%7B")
            string = string.replace("}", "%7D")
            string = string.replace("|", "%7C")
            return string

        allowed_filters = ["creationdate", "lastmodifieddate", "orderfulfillmentstatus"]
        allowed_orderfulfillmentstatuses = [
            "{NOT_STARTED|IN_PROGRESS}",
            "{FULFILLED|IN_PROGRESS}",
        ]
        for pair in filter_string.split(","):
            key, value = pair.split(":")[0], ":".join(pair.split(":")[1:])

            if key == "orderfulfillmentstatus":
                if value.strip() not in allowed_orderfulfillmentstatuses:
                    self.exception_class(
                        f"Недопустимое значение фильтра {key}: {value}. Допустимые значения: {allowed_orderfulfillmentstatuses}.\n"
                    )
            elif key in ["creationdate", "lastmodifieddate"]:
                pass
                # TODO: проверить на соответствие шаблону YYYY-MM-DDThh:mm:ss.000Z
                # if not is_datetime(...):
                #     message += f"Недопустимое значение фильтра {key}: {value}.\n"
                #     message += f"Значение должно соответствовать шаблону: [<datetime>..<datetime or empty string>]"
            else:
                self.exception_class(f"Недопустимый фильтр: {key}. Допустимые фильтры: {allowed_filters}.\n")

        return ",".join([_percent_encode(filter_pair.strip()) for filter_pair in filter_string.split(",")])


class IssueRefund(EbayAPI):
    """
    Docs:
    https://developer.ebay.com/api-docs/sell/fulfillment/resources/order/methods/issueRefund
    """

    http_method = "POST"
    resource_method = "sell/fulfillment/v1/order/{orderId}/issue_refund"
    required_params = ["orderId"]



from model_utils import Choices

from django.utils.translation import gettext_lazy as _


YOOKASSA_PAYMENT_STATUSES = Choices(
    ("pending", _("Ожидание подтверждения")),
    ("succeeded", _("Успешно выполнено")),
    ("canceled", _("Отменено")),
    ("waiting_for_capture", _("Ожидание списания")),
)

YOOKASSA_PAYMENT_CANCELLATION_PARTIES = Choices(
    ("yandex_checkout", _("Яндекс.Касса")),
    ("payment_network", _("Внешние участники платежного процесса")),
    ("merchant", _("Продавец товаров и услуг")),
)

YOOKASSA_CANCELLATION_REASONS = Choices(
    ("3d_secure_failed", _("Не пройдена аутентификация по 3-D Secure")),
    ("call_issuer", _("Оплата данным платежным средством отклонена по неизвестным причинам")),
    ("canceled_by_merchant", _("Платеж отменен по API при оплате в две стадии")),
    ("card_expired", _("Истек срок действия банковской карты")),
    ("country_forbidden", _("Нельзя заплатить банковской картой, выпущенной в этой стране")),
    ("expired_on_capture", _("Истек срок списания оплаты у двухстадийного платежа")),
    (
        "expired_on_confirmation",
        _("Истек срок оплаты: пользователь не подтвердил платеж за время, отведенное на оплату выбранным способом"),
    ),
    ("fraud_suspected", _("Платеж заблокирован из-за подозрения в мошенничестве")),
    ("general_decline", _("Причина не детализирована")),
    ("identification_required", _("Превышены ограничения на платежи для кошелька в Яндекс.Деньгах")),
    ("insufficient_funds", _("Не хватает денег для оплаты")),
    (
        "internal_timeout",
        _("Технические неполадки на стороне Яндекс.Кассы: не удалось обработать запрос в течение 30 секунд"),
    ),
    ("invalid_card_number", _("Неправильно указан номер карты")),
    ("invalid_csc", _("Неправильно указан код CVV2 (CVC2, CID)")),
    ("issuer_unavailable", _("Организация, выпустившая платежное средство, недоступна")),
    ("payment_method_limit_exceeded", _("Исчерпан лимит платежей для данного платежного средства или магазина")),
    ("payment_method_restricted", _("Запрещены операции данным платежным средством")),
    ("permission_revoked", _("Нельзя провести безакцептное списание: пользователь отозвал разрешение на автоплатежи")),
)

YOOKASSA_RECEIPT_REGISTRATION_STATUSES = Choices(
    ("pending", _("Ожидание")),
    ("succeeded", _("Успешно выполнено")),
    ("canceled", _("Отменено")),
)

YOOKASSA_RECEIPT_TYPES = Choices("payment", "refund")

YOOKASSA_TAX_SYSTEM_CODES = Choices(
    (1, "n1", "Общая система налогообложения"),
    (2, "n2", "Упрощенная (УСН, доходы)"),
    (3, "n3", "Упрощенная (УСН, доходы минус расходы)"),
    (4, "n4", "Единый налог на вмененный доход (ЕНВД)"),
    (5, "n5", "Единый сельскохозяйственный налог (ЕСН)"),
    (6, "n6", "Патентная система налогообложения"),
)

YOOKASSA_VAT_CODES = Choices(
    (1, "n1", "Без НДС"),
    (2, "n2", "НДС по ставке 0%"),
    (3, "n3", "НДС по ставке 10%"),
    (4, "n4", "НДС чека по ставке 20%"),
    (5, "n5", "НДС чека по расчетной ставке 10/110"),
    (6, "n6", "НДС чека по расчетной ставке 20/120"),
)

YOOKASSA_PAYMENT_SUBJECTS = Choices(
    ("service", "Услуга"),
)

YOOKASSA_PAYMENT_MODES = Choices(
    ("full_prepayment", "Полная предоплата"),
    ("partial_prepayment", "Частичная предоплата"),
    ("advance", "Аванс"),
    ("full_payment", "Полный расчет"),
    ("partial_payment", "Частичный расчет и кредит"),
    ("credit", "Кредит"),
    ("credit_payment", "Выплата по кредиту"),
)

YOOKASSA_SETTLEMENT_TYPES = Choices(
    ("cashless", "Безналичный расчет"),
    ("prepayment", "Предоплата (аванс)"),
    ("postpayment", "Постоплата (кредит)"),
    ("consideration", "Встречное предоставление"),
)

YOOKASSA_AGENT_TYPES = Choices(
    ("banking_payment_agent", "Банковский платежный агент"),
    ("banking_payment_subagent", "Банковский платежный субагент"),
    ("payment_agent", "Платежный агент"),
    ("payment_subagent", "Платежный субагент"),
    ("attorney", "Поверенный"),
    ("commissioner", "Комиссионер"),
    ("agent", "Агент"),
)

YOOKASSA_PURPOSE = Choices(
    ("CARD_SETUP", "Подключение карты"),
)



from model_utils import Choices

from django.utils.translation import gettext_lazy as _


__all__ = [
    "USER_LANGUAGES",
    "MARKETPLACE_LANGUAGES",
    "OTHER_LANGUAGES",
    "LANGUAGES",
]


USER_LANGUAGES = Choices(
    ("ru", _("Русский")),
    ("en", _("Английский")),
)


MARKETPLACE_LANGUAGES = Choices(
    ("en", _("Английский")),
    ("ru", _("Русский")),
    ("de", _("Немецкий")),
    ("zh", _("Китайский")),
    ("pl", _("Польский")),
    ("fr", _("Французский")),
    ("it", _("Итальянский")),
    ("pt", _("Португальский")),
    ("es", _("Испанский")),
    ("ja", _("Японский")),
    ("th", _("Тайский")),
    ("nl", _("Голландский")),
)


OTHER_LANGUAGES = Choices(
    ("ab", _("Абхазский")),
    ("aa", _("Афар")),
    ("af", _("Африкаанс")),
    ("ak", _("Акан")),
    ("sq", _("Албанский")),
    ("am", _("Амхарский")),
    ("ar", _("Арабский")),
    ("an", _("Арагонский")),
    ("hy", _("Армянский")),
    ("as", _("Ассамский")),
    ("av", _("Аварский")),
    ("ae", _("Авестийский")),
    ("ay", _("Аймара")),
    ("az", _("Азербайджанский")),
    ("bm", _("Бамбара")),
    ("ba", _("Башкирский")),
    ("eu", _("Баскский")),
    ("be", _("Белорусский")),
    ("bn", _("Бенгальский")),
    ("bh", _("Бихари")),
    ("bi", _("Бислама")),
    ("bs", _("Боснийский")),
    ("br", _("Бретонский")),
    ("bg", _("Болгарский")),
    ("my", _("Бирманский")),
    ("ca", _("Каталанский")),
    ("ch", _("Чаморро")),
    ("ce", _("Чеченский")),
    ("ny", _("Ньянджа")),
    ("cv", _("Чувашский")),
    ("kw", _("Корнский")),
    ("co", _("Корсиканский")),
    ("cr", _("Кри")),
    ("hr", _("Хорватский")),
    ("cs", _("Чешский")),
    ("da", _("Датский")),
    ("dv", _("Дивехи")),
    ("dz", _("Дзонг-Кэ")),
    ("eo", _("Эсперанто")),
    ("et", _("Эстонский")),
    ("ee", _("Эве")),
    ("fo", _("Фарерский")),
    ("fj", _("Фиджи")),
    ("fi", _("Финский")),
    ("ff", _("Фулах")),
    ("gl", _("Галисийский")),
    ("ka", _("Грузинский")),
    ("el", _("Новогреческий (1453-)")),
    ("gn", _("Гуарани")),
    ("gu", _("Гуджарати")),
    ("ht", _("Гаитянский")),
    ("ha", _("Хауса")),
    ("he", _("Иврит")),
    ("hz", _("Гереро")),
    ("hi", _("Хинди")),
    ("ho", _("Хири Моту")),
    ("hu", _("Венгерский")),
    ("ia", _("Интерлингва (Ассоциация Международного Вспомогательного Языка)")),
    ("id", _("Индонезийский")),
    ("ie", _("Интерлингве")),
    ("ga", _("Ирландский")),
    ("ig", _("Игбо")),
    ("ik", _("Инулиак")),
    ("io", _("Идо")),
    ("is", _("Исландский")),
    ("iu", _("Инуктитут")),
    ("jv", _("Яванский")),
    ("kl", _("Калааллисут")),
    ("kn", _("Каннада")),
    ("kr", _("Канури")),
    ("ks", _("Кашмири")),
    ("kk", _("Казахский")),
    ("km", _("Кхмерский Центральный")),
    ("ki", _("Кикуйю")),
    ("rw", _("Киньяруанда")),
    ("ky", _("Киргизский")),
    ("kv", _("Коми")),
    ("kg", _("Конго")),
    ("ko", _("Корейский")),
    ("ku", _("Курдский")),
    ("kj", _("Киньяма")),
    ("la", _("Латынь")),
    ("lb", _("Люксембургский")),
    ("lg", _("Ганда")),
    ("li", _("Лимбурган")),
    ("ln", _("Лингала")),
    ("lo", _("Лаосский")),
    ("lt", _("Литовский")),
    ("lu", _("Луба-Катанга")),
    ("lv", _("Латышский")),
    ("gv", _("Мэнкский")),
    ("mk", _("Македонский")),
    ("mg", _("Малагасийский")),
    ("ms", _("Малайский (Макроязык)")),
    ("ml", _("Малаялам")),
    ("mt", _("Мальтийский")),
    ("mi", _("Маори")),
    ("mr", _("Маратхи")),
    ("mh", _("Маршалльский")),
    ("mn", _("Монгольский")),
    ("na", _("Науру")),
    ("nv", _("Навахо")),
    ("nb", _("Норвежский Букмол")),
    ("nd", _("Северный Ндебеле")),
    ("ne", _("Непальский (макроязык)")),
    ("ng", _("Ндунга")),
    ("nn", _("Норвежский Нюнорск")),
    ("no", _("Норвежский")),
    ("ii", _("Сычуань Йи")),
    ("nr", _("Южный Ндебеле")),
    ("oc", _("Окситанский (пост 1500)")),
    ("oj", _("Оджибва")),
    ("cu", _("Церковнославянский")),
    ("om", _("Оромо")),
    ("or", _("Ория (макроязык)")),
    ("os", _("Осетинский")),
    ("pa", _("Панджаби")),
    ("pi", _("Пали")),
    ("fa", _("Персидский")),
    ("ps", _("Пушту")),
    ("qu", _("Кечуа")),
    ("rm", _("Ретороманский")),
    ("rn", _("Рунди")),
    ("ro", _("Румынский")),
    ("sa", _("Санскрит")),
    ("sc", _("Сардинский")),
    ("sd", _("Синдхи")),
    ("se", _("Северные Саамы")),
    ("sm", _("Самоанский")),
    ("sg", _("Санго")),
    ("sr", _("Сербский")),
    ("gd", _("Шотландский Гэльский")),
    ("sn", _("Шона")),
    ("si", _("Сингальский")),
    ("sk", _("Словацкий")),
    ("sl", _("Словенский")),
    ("so", _("Сомали")),
    ("st", _("Южный Сото")),
    ("su", _("Сунданский")),
    ("sw", _("Суахили (Макроязык)")),
    ("ss", _("Свати")),
    ("sv", _("Шведский")),
    ("ta", _("Тамильский")),
    ("te", _("Телугу")),
    ("tg", _("Таджикский")),
    ("ti", _("Тигринья")),
    ("bo", _("Тибетский")),
    ("tk", _("Туркменский")),
    ("tl", _("Тагальский")),
    ("tn", _("Тсвана")),
    ("to", _("Тонга (Острова Тонга)")),
    ("tr", _("Турецкий")),
    ("ts", _("Тсонга")),
    ("tt", _("Татарский")),
    ("tw", _("Тви")),
    ("ty", _("Таитянский")),
    ("ug", _("Уйгурский")),
    ("uk", _("Украинский")),
    ("ur", _("Урду")),
    ("uz", _("Узбекский")),
    ("ve", _("Венда")),
    ("vi", _("Вьетнамский")),
    ("vo", _("Волапюк")),
    ("wa", _("Валлун")),
    ("cy", _("Валлийский")),
    ("wo", _("Волоф")),
    ("fy", _("Западно-фризский")),
    ("xh", _("Коса")),
    ("yi", _("Идиш")),
    ("yo", _("Йоруба")),
    ("za", _("Чжуан")),
    ("zu", _("Зулусский")),
)

LANGUAGES = MARKETPLACE_LANGUAGES + OTHER_LANGUAGES



from zs_utils.api.wildberries.base_api import WildberriesAPI


class GetWildberriesWarehouseList(WildberriesAPI):
    """
    https://openapi.wildberries.ru/?ysclid=ljzow9j5e0958977292#tag/Marketplace-Sklady/paths/~1api~1v3~1warehouses/get
    """

    http_method = "GET"
    resource_method = "api/v3/warehouses"


class GetWildberriesWarehouseStock(WildberriesAPI):
    """
    https://openapi.wb.ru/#tag/Marketplace-Ostatki/paths/~1api~1v3~1stocks~1%7BwarehouseId%7D/post
    """

    http_method = "POST"
    resource_method = "api/v3/stocks/{warehouseId}"
    required_params = ["skus"]


class DeleteWildberriesStocks(WildberriesAPI):
    """
    https://openapi.wb.ru/#tag/Marketplace-Ostatki/paths/~1api~1v3~1stocks~1%7BwarehouseId%7D/delete
    """

    http_method = "DELETE"
    resource_method = "api/v3/stocks/{warehouseId}"
    required_params = ["skus"]


class UpdateWildberriesStocks(WildberriesAPI):
    """
    https://openapi.wb.ru/#tag/Marketplace-Ostatki/paths/~1api~1v3~1stocks~1%7BwarehouseId%7D/put
    """

    http_method = "PUT"
    resource_method = "api/v3/stocks/{warehouseId}"
    required_params = ["stocks"]



import re


__all__ = [
    "custom_quote",
    "custom_unquote",
]


mapping = {
    ";": "a",
    "/": "b",
    "?": "c",
    ":": "d",
    "@": "e",
    "&": "f",
    "=": "g",
    "+": "h",
    "$": "i",
    ",": "j",
    "~": "k",
}


def custom_quote(value: str):
    if not value:
        return value

    def repl(matchobj):
        return f"_ZS1{mapping[matchobj.group(0)]}ZS2_"

    pattern = "[" + "".join(mapping.keys()) + "]"

    return re.sub(pattern=pattern, repl=repl, string=str(value))


def custom_unquote(value: str):
    if not value:
        return value

    def repl(matchobj):
        for key, value in mapping.items():
            if value == matchobj.group(1):
                return key
        raise ValueError()

    pattern = "_ZS1([" + "".join(mapping.values()) + "])ZS2_"

    return re.sub(pattern=pattern, repl=repl, string=str(value))



import os
import io
import base64
import ssl
from urllib.request import urlopen, Request
from urllib.parse import urlparse
from uuid import uuid4


__all__ = [
    "open_url_with_basic_auth",
    "get_file_by_url",
    "get_url_file_ext",
    "generate_unique_name_for_url_file",
]


def open_url_with_basic_auth(url: str, username: str, password: str):
    request = Request(url=url)
    b64auth = base64.b64encode(f"{username}:{password}".encode())
    request.add_header("Authorization", f"Basic {b64auth.decode()}")
    return urlopen(request, context=ssl.SSLContext(protocol=ssl.PROTOCOL_TLS))


def get_file_by_url(url: str, auth_username: str = None, auth_password: str = None) -> io.BytesIO:
    file = io.BytesIO()

    if auth_username and auth_password:
        page = open_url_with_basic_auth(url=url, username=auth_username, password=auth_password)
    else:
        page = urlopen(
            url=Request(url=url, headers={"User-Agent": "Magic Browser"}),
            context=ssl.SSLContext(protocol=ssl.PROTOCOL_TLS),
        )
    page.getcode()

    file.write(page.read())
    file.seek(0)

    return file


def get_url_file_ext(url: str) -> str:
    path = urlparse(url).path
    return os.path.splitext(path)[1]


def generate_unique_name_for_url_file(url: str) -> str:
    name = str(uuid4())

    ext = get_url_file_ext(url)
    if ext:
        name += ext

    return name



from rest_framework import serializers
from rest_framework.exceptions import ValidationError

from zs_utils.base_support import constants, models
from zs_utils.user.serializers import UserLightSerializer


__all__ = [
    "CommonSupportTicketSerializer",
    "CommonCreateSupportTicketSerializer",
    "CommonSupportTicketMessageSerializer",
    "CommonCreateSupportTicketMessageSerializer",
    "CommonSupportTicketMessageFileSerializer",
]


class CommonSupportTicketSerializer(serializers.ModelSerializer):
    user = UserLightSerializer()
    manager = UserLightSerializer(read_only=True)

    class Meta:
        model = models.SupportTicket
        fields = [
            "id",
            "number",
            "created",
            "user",
            "manager",
            "status",
            "question_type",
            "subject",
            "unread_messages",
            "client_status"
        ]


class CommonSupportTicketMessageFileSerializer(serializers.ModelSerializer):
    class Meta:
        model = models.SupportTicketMessageFile
        fields = [
            "id",
            "ticket_message",
            "file",
            "user",
            "name",
        ]


class CommonSupportTicketMessageSerializer(serializers.ModelSerializer):
    sender = UserLightSerializer(read_only=True)
    files = CommonSupportTicketMessageFileSerializer(many=True)

    class Meta:
        model = models.SupportTicketMessage
        fields = [
            "id",
            "created",
            "ticket",
            "sender",
            "text",
            "files",
            "is_system",
            "is_viewed",
        ]


class CommonSupportTicketWithMessagesSerializer(CommonSupportTicketSerializer):
    messages = CommonSupportTicketMessageSerializer(many=True, read_only=True)

    class Meta(CommonSupportTicketSerializer.Meta):
        model = models.SupportTicket
        fields = CommonSupportTicketSerializer.Meta.fields + ["messages"]

    def to_representation(self, instance):
        data = super().to_representation(instance)

        data["user"] = UserLightSerializer(instance.user).data
        data["user"]["phone"] = instance.user.phone

        return data


class CommonCreateSupportTicketMessageSerializer(serializers.Serializer):
    text = serializers.CharField(required=False, allow_null=True)
    files = serializers.ListField(
        child=serializers.PrimaryKeyRelatedField(queryset=models.SupportTicketMessageFile.objects.all()),
        required=False,
        max_length=constants.MAX_TICKET_MESSAGE_FILES,
    )

    def to_internal_value(self, data):
        data = super().to_internal_value(data)

        if not (data.get("text") or data.get("files")):
            raise ValidationError({"text": "Обязательное поле, если не задано поле 'files'."})

        return data


class CommonCreateSupportTicketSerializer(serializers.ModelSerializer):
    message = CommonCreateSupportTicketMessageSerializer(required=False)

    class Meta:
        model = models.SupportTicket
        fields = [
            "question_type",
            "subject",
            "message",
        ]

    def to_internal_value(self, data):
        if "message" in data and data["message"] is not None:
            message_serializer = self.fields["message"]
            message_internal = message_serializer.to_internal_value(data["message"])
            data["message"] = message_internal
        elif "message" in data and data["message"] is None:
            del data["message"]

        return super().to_internal_value(data)



from zs_utils.api.wildberries.base_api import WildberriesAPI


class WildberriesPriceAPI(WildberriesAPI):
    production_api_url = "https://discounts-prices-api.wb.ru/"


class GetWildberriesPrices(WildberriesPriceAPI):
    """
    https://openapi.wb.ru/prices/api/ru/#tag/Spiski-tovarov/paths/~1api~1v2~1list~1goods~1filter/get
    """

    http_method = "GET"
    resource_method = "api/v2/list/goods/filter"
    required_params = ["limit"]
    allowed_params = ["offset", "filterNmID"]


class UpdateWildberriesPrices(WildberriesPriceAPI):
    """
    https://openapi.wb.ru/prices/api/ru/#tag/Ustanovka-cen-i-skidok/paths/~1api~1v2~1upload~1task/post
    """

    http_method = "POST"
    resource_method = "api/v2/upload/task"
    required_params = ["data"]



from .billing import *
from .category import *
from .best_offer import *
from .feedback import *
from .listing import *
from .details import *
from .messages import *
from .notifications import *
from .limits import *
from .image import *
from .store import *



import datetime

from zs_utils.api.ebay.base_api import EbayTradingAPI


__all__ = [
    "GetListingList",
    "GetListing",
]


class GetListingList(EbayTradingAPI):
    """
    Docs:
    https://developer.ebay.com/devzone/xml/docs/reference/ebay/GetSellerList.html
    """

    method_name = "GetSellerList"

    def get_params(
        self, PageNumber: int, EntriesPerPage: int = 200, EndTimeFrom=None, EndTimeTo=None, CategoryID=None, **kwargs
    ):
        if isinstance(EndTimeTo, str):
            EndTimeTo = datetime.datetime.fromisoformat(EndTimeTo)
        if isinstance(EndTimeFrom, str):
            EndTimeFrom = datetime.datetime.fromisoformat(EndTimeFrom)
        if not EndTimeFrom:
            EndTimeFrom = datetime.datetime.now()
        if (not EndTimeTo) or ((EndTimeTo - EndTimeFrom).days > 120):
            EndTimeTo = EndTimeFrom + datetime.timedelta(days=120)

        params = {
            "CategoryID": CategoryID,
            "EndTimeFrom": EndTimeFrom.isoformat(),
            "EndTimeTo": EndTimeTo.isoformat(),
            "GranularityLevel": "Medium",
            "IncludeVariations": True,
            "DetailLevel": "ItemReturnDescription",
            "WarningLevel": "High",
            "Pagination": {
                "PageNumber": PageNumber,
                "EntriesPerPage": EntriesPerPage,
            },
        }

        return params


class GetListing(EbayTradingAPI):
    """
    Docs:
    https://developer.ebay.com/devzone/xml/docs/Reference/eBay/GetItem.html
    """

    method_name = "GetItem"

    def get_params(self, remote_id: str, detail_level="ReturnAll", **kwargs):
        return {
            "ItemID": remote_id,
            "IncludeItemCompatibilityList": True,
            "IncludeItemSpecifics": True,
            "DetailLevel": detail_level,
        }



from model_utils import Choices


CRM_LANDING_STAGES = Choices(
    ("C22:UC_GR0G8F", "NEW", "Обработка"),
    ("C22:UC_44PJ08", "IDENTIFY_NEEDS", "Выявление потребностей"),
    ("C22:NEW", "BRIEFING", "Брифинг"),
    ("C22:PREPARATION", "PREPARING", "Подготовка стратегии / КП"),
    ("C22:PREPAYMENT_INVOIC", "PRESENTED", "Стратегия / КП - отправлено (презентовано)"),
    ("C22:EXECUTING", "NEGOTIATION", "Переговоры по КП"),
    ("C22:FINAL_INVOICE", "DECISION_MAKING", "Принятие решения"),
    ("C22:UC_QP7BZQ", "CONTRACT_SIGNING", "Подписание договора"),
    ("C22:UC_XJPR5R", "CONTRACT_SIGNED", "Договор подписан"),
    ("C22:UC_4Z82CF", "ACCOUNTING", "Передача на аккаунтинг"),
    ("C22:UC_6ZM2Q5", "PREPARE_LAUNCH", "Подготовка к запуску"),
    ("C22:UC_Z9RIPX", "LAUNCHING", "Запуск"),
    ("C22:UC_8WBK1X", "LAUNCHED", "Запуск завершён"),
    ("C22:APOLOGY", "ARCHIVED", "Архив"),
    ("C22:WON", "SUPPORT", "Поддержка клиента"),
    ("C22:LOSE", "END", "Завершение сотрудничества"),
)

CRM_OBJECTS = Choices(
    ("CONTACT", "Контакт"),
    ("DEAL", "Сделка"),
    ("STATUS", "Статус"),
)

CRM_ACTIVITY_TYPES = Choices(
    (1, "MEETING", "Встреча"),
    (2, "CALL", "Звонок"),
    (3, "TASK", "Задача"),
    (4, "MESSAGE", "Письмо"),
    (5, "ACTION", "Действие"),
)



from abc import abstractmethod
from ebaysdk.exception import ConnectionError as BaseEbayTradingAPIError
from ebaysdk.trading import Connection as BaseEbayTradingAPI
from ebaysdk.response import Response as EbaySDKResponse

from zs_utils.api.base_api import BaseAPI
from zs_utils.api.ebay.data.marketplace.marketplace_to_site import (
    EbayDomainCodeToSiteID,
)


__all__ = [
    "EbayTradingAPI",
]


class EbayTradingAPI(BaseAPI):
    def __init__(
        self,
        access_token: str,
        client_id: str,
        client_secret: str,
        dev_id: str,
        site_id: str = None,
        domain_code: str = None,
        debug: bool = False,
        **kwargs,
    ):
        super().__init__(**kwargs)

        self.access_token = access_token
        self.client_id = client_id
        self.client_secret = client_secret
        self.dev_id = dev_id
        self.site_id = site_id
        self.domain_code = domain_code
        self.debug = debug

    def validate_attrs(self) -> None:
        pass

    @property
    @abstractmethod
    def method_name(self) -> str:
        pass

    @abstractmethod
    def get_params(self, **kwargs) -> dict:
        pass

    def _clean_data(self, data: dict) -> dict:
        cleaned_data = {}
        for key, value in data.items():
            if value or type(value) in [bool, int]:
                if isinstance(value, dict):
                    cleaned_data.update({key: self._clean_data(value)})
                else:
                    cleaned_data.update({key: value})
        return cleaned_data

    def get_site_id(self, site_id: str = None, domain_code: str = None) -> str:
        if site_id:
            if site_id not in EbayDomainCodeToSiteID.values():
                raise BaseEbayTradingAPIError(msg=f"Неизвестный 'site_id'={site_id}")
        elif domain_code:
            if domain_code not in EbayDomainCodeToSiteID.keys():
                raise BaseEbayTradingAPIError(msg=f"Маркетплейс '{domain_code}' не поддерживается Trading API.")
            else:
                site_id = EbayDomainCodeToSiteID[domain_code]
        else:
            site_id = EbayDomainCodeToSiteID["default"]

        return site_id

    def make_request(self, **kwargs) -> EbaySDKResponse:
        params = self.get_params(**kwargs)
        cleaned_params = self._clean_data(params)

        api = BaseEbayTradingAPI(
            iaf_token=self.access_token,
            appid=self.client_id,
            devid=self.dev_id,
            certid=self.client_secret,
            siteid=self.get_site_id(site_id=self.site_id, domain_code=self.domain_code),
            config_file=None,
            debug=self.debug,
        )
        return api.execute(
            verb=self.method_name,
            data=cleaned_params,
        )



from xml.dom import minidom
from xml.etree import ElementTree as ET

from zs_utils.yandex_market_language.models import Feed


class YML:
    """
    Main class for feed parse and conversion.
    """

    def __init__(self, file_or_path):
        self._file_or_path = file_or_path

    @staticmethod
    def prettify_el(el: "ET.Element") -> "ET.Element":
        """
        Return a pretty-printed XML string for the Element.
        """
        raw = ET.tostring(el, "utf-8")
        parsed = minidom.parseString(raw)
        prettified = parsed.toprettyxml(indent="\t")
        return ET.fromstring(prettified)

    def parse(self) -> "Feed":
        """
        Parses an XML feed file to the Feed model.
        """
        tree = ET.parse(self._file_or_path)
        root = tree.getroot()
        return Feed.from_xml(root)

    def convert(self, feed: "Feed", pretty: bool = True):
        """
        Converts Feed model to XML file.
        """
        feed_el = feed.to_xml()
        if pretty:
            feed_el = self.prettify_el(feed_el)
        tree = ET.ElementTree(feed_el)
        tree.write(self._file_or_path, encoding="utf-8")


def parse(file_or_path):
    return YML(file_or_path).parse()


def convert(file_or_path, feed: "Feed", pretty: bool = True):
    YML(file_or_path).convert(feed, pretty)



from distutils.util import strtobool
from rest_framework import status
from rest_framework.decorators import action
from rest_framework.response import Response
from rest_framework.exceptions import ValidationError
from rest_framework.views import View
from rest_framework.serializers import Serializer

from django.conf import settings
from django.contrib.auth.models import AnonymousUser
from django.contrib.auth import get_user_model
from django.utils.translation import gettext as _


__all__ = [
    "AdminModeViewMixin",
    "ResponseShortcutsViewMixin",
    "DataValidationViewMixin",
    "OnlyInstanceViewMixin",
    "NearbyIdsViewMixin",
]


class AdminModeViewMixin(View):
    def _user_is_staff(self, user: settings.AUTH_USER_MODEL) -> bool:
        return user.is_staff

    @property
    def admin_mode(self) -> bool:
        """
        Включен режим администратора
        """
        if not getattr(self, "_admin_mode", None):
            return self._user_is_staff(user=self.request.user) and bool(
                strtobool(self.request.GET.get("admin_mode", "false"))
            )
        return self._admin_mode

    @admin_mode.setter
    def admin_mode(self, value: bool) -> None:
        """
        Установка значения для режима администрации
        """
        self._admin_mode = value

    @property
    def no_limit(self) -> bool:
        """
        Включён режим без ограничений
        """
        if not getattr(self, "_no_limit", None):
            self._no_limit = self.admin_mode and bool(strtobool(self.request.GET.get("no_limit", "false")))
        return self._no_limit

    @no_limit.setter
    def no_limit(self, value) -> None:
        """
        Установка значения для режима без ограничений
        """
        self._no_limit = value

    def get_user(self):
        """
        Получение пользователя user.
        В режиме администрации можно указывать любого пользователя и действовать от его лица
        """
        if self.admin_mode:
            user_id = self.request.GET.get("user_id")
            if user_id:
                User = get_user_model()
                try:
                    return User.objects.get(id=user_id)
                except User.DoesNotExist:
                    raise ValidationError({"user_id": _("Пользователь не найден.")})

        user = self.request.user
        if isinstance(user, AnonymousUser):
            user = None

        return user


class OnlyInstanceViewMixin(View):
    """
    View-mixin для работы с единичными объектами (к пример у User лишь один объект настроек UserSettings)
    """

    def get_object(self):
        """
        Получение единственного объекта
        """
        return self.get_queryset().first()

    def list(self, request, *args, **kwargs):
        """
        Замена отдачи списка объектов на единичный объект
        """
        if getattr(self, "no_limit", False):
            return super().list(request, *args, **kwargs)

        instance = self.get_object()
        if instance:
            data = self.get_serializer(instance).data
        else:
            data = {}

        return self.build_response(data=data)

    @action(detail=False, methods=["PUT", "PATCH"])
    def alter(self, request, *args, **kwargs):
        """
        Обновление объекта
        """
        instance = self.get_object()

        serializer = self.get_serializer(instance=instance, data=request.data)
        serializer.is_valid(raise_exception=True)
        instance = serializer.save()

        retrieve_serializer = self.serializer_classes.get(
            "retrieve", self.serializer_classes.get("default", self.serializer_class)
        )
        return self.build_response(data=retrieve_serializer(instance=instance).data)


class NearbyIdsViewMixin(View):
    """
    View-mixin для получения соседних идентификаторов
    """

    @action(detail=True, methods=["GET"])
    def get_nearby_ids(self, request, *args, **kwargs):
        """
        Получение соседних идентификаторов относительно переданного объекта
        """
        if hasattr(self, "actions_for_filterset_class"):
            if self.actions_for_filterset_class != "__all__":
                self.actions_for_filterset_class += ["get_nearby_ids"]

        qs = self.filter_queryset(self.get_queryset())
        ids = list(qs.values_list(self.lookup_field, flat=True))
        obj_id = getattr(self.get_object(), self.lookup_field)
        obj_index = ids.index(obj_id)
        prev_id = ids[obj_index - 1] if (obj_index - 1 >= 0) else None
        next_id = ids[obj_index + 1] if (obj_index + 1 <= qs.count() - 1) else None

        return Response({"prev_id": prev_id, "next_id": next_id}, status=status.HTTP_200_OK)


class ResponseShortcutsViewMixin(View):
    def build_response(
        self,
        message: str = None,
        data: dict = None,
        status_code: int = status.HTTP_200_OK,
        **kwargs,
    ) -> Response:
        """
        Создать ответ с переданным сообщением или данными. (Положительный по умолчанию)
        """

        if not data:
            if message:
                data = {"message": message}
        elif message:
            if isinstance(data, dict):
                data = {"message": message, **data}
            else:
                data = {"message": message, "data": data}

        return Response(data=data, status=status_code)

    def build_error_response(
        self,
        message: str = None,
        data: dict = None,
        status_code: int = status.HTTP_400_BAD_REQUEST,
        **kwargs,
    ) -> Response:
        """
        Создать ответ с ошибкой с переданным сообщением или данными
        """
        return self.build_response(
            message=message,
            data=data,
            status_code=status_code,
            **kwargs,
        )


class DataValidationViewMixin(View):
    def validate_data(self, serializer_class: Serializer, data: dict = None) -> Serializer:
        """
        Валидация данных по переданному сериализатору
        """

        if not data:
            data = {}
            if isinstance(self.request.data, dict):
                data.update(self.request.data)
            data.update(self.request.GET.dict())

        serializer = serializer_class(data=data)
        serializer.is_valid(raise_exception=True)
        return serializer

    def get_validated_data(self, serializer_class, data: dict = None) -> dict:
        """
        Получение провалидированных данных по переданному сериализатору
        """
        serializer = self.validate_data(
            serializer_class=serializer_class,
            data=data,
        )
        return serializer.validated_data



from django.contrib.auth.tokens import PasswordResetTokenGenerator
from django.utils.encoding import force_bytes, force_str
from django.utils.http import urlsafe_base64_decode, urlsafe_base64_encode


def encode_uid(pk):
    return force_str(urlsafe_base64_encode(force_bytes(pk)))


def decode_uid(pk):
    return force_str(urlsafe_base64_decode(pk))


class PasswordResetTokenGeneratorIgnoreLastLogin(PasswordResetTokenGenerator):
    def _make_hash_value(self, user, timestamp):
        return str(user.pk) + user.password + str(timestamp)


ignore_last_login_token_generator = PasswordResetTokenGeneratorIgnoreLastLogin()



import threading
import _thread


def quit_function():
    """
    Прерывание выполнения функции исключением KeyboardInterrupt.
    """

    _thread.interrupt_main()


def exit_after_timeout(timeout: float):
    """
    Декоратор функции для ограничения времени ее работы.
    """

    def outer(func):
        def inner(*args, **kwargs):
            timer = threading.Timer(interval=timeout, function=quit_function)
            try:
                timer.start()
                result = func(*args, **kwargs)
            except KeyboardInterrupt:
                raise TimeoutError("Выполнение функции длилось слишком долго.")
            finally:
                timer.cancel()
            return result

        return inner

    return outer



from zs_utils.api.ebay.base_api import EbayTradingAPI


__all__ = [
    "GetFeedbackList",
]


class GetFeedbackList(EbayTradingAPI):
    """
    Docs:
    https://developer.ebay.com/devzone/xml/docs/reference/ebay/GetFeedback.html
    """

    method_name = "GetFeedback"

    def get_params(self, **kwargs):
        return {
            "DetailLevel": "ReturnAll",
            "Pagination": {
                "EntriesPerPage": 200,
                "PageNumber": kwargs["page"],
            },
        }



import logging
import requests
import re
from abc import ABC


__all__ = [
    "BaseAPI",
]


class BaseAPI(ABC):
    required_params = []
    allowed_params = []

    # Тело запроса -- список, а не словарь
    array_payload = False

    # Отбрасывать параметры с пустым значением (None или "")
    drop_empty_params = False

    http_method = None
    resource_method = None
    production_api_url = None
    sandbox_api_url = None

    def __init__(
        self,
        logger=None,
        is_sandbox: bool = False,
        response_timeout: float = 30,
        **kwargs,
    ):
        self.logger = logger or logging.getLogger(self.__class__.__name__)
        self.is_sandbox = is_sandbox
        self.response_timeout = response_timeout

        self.validate_attrs()

    def validate_attrs(self) -> None:
        """
        Валидация обязательных атрибутов у дочерних классов
        """
        for attr in [
            "http_method",
            "resource_method",
            "production_api_url",
        ]:
            if not getattr(self, attr, None):
                raise ValueError(f"Необходимо определить атрибут '{attr}'.")

        if self.is_sandbox and (not self.sandbox_api_url):
            raise ValueError("Для использования тестового режима необходимо указать 'sandbox_api_url'")

        if self.array_payload:
            if (self.required_params and (self.required_params != ["array_payload"])) or self.allowed_params:
                raise ValueError(
                    "Если стоит флаг 'array_payload', то единственным возможным и обязательным параметром является параметр 'array_payload'."
                )
            self.required_params = ["array_payload"]

    @property
    def headers(self) -> dict:
        """
        Headers для запросов к API
        """
        return {}

    @property
    def path_params(self) -> list[str]:
        return re.findall(pattern=r"\{([^\}]+)\}", string=self.resource_method)

    def build_url(self, params: dict) -> str:
        """
        Составление url для запросов к API (подстановка эндпоинта и url-параметром в url)
        """
        url = self.sandbox_api_url if self.is_sandbox else self.production_api_url
        url += self.resource_method

        path_params = {}
        for param in self.path_params:
            if params.get(param):
                path_params[param] = params[param]
            else:
                raise ValueError(f"Отсутствует обязательный параметр '{param}'.")

        if path_params:
            url = url.format(**path_params)
            if url[-1] == "/":
                url = url[:-1]
        return url

    def get_clean_params(self, params: dict) -> dict:
        """
        Отчистка и валидация параметров запроса
        В итоговый запрос попадут только ключи из required_params и allowed_param
        """
        clean_params = {}
        files = {}

        for req_param in self.required_params:
            if req_param not in params:
                raise ValueError(f"Обязательный параметр запроса '{req_param}' не задан.")

            if isinstance(params[req_param], bytes):
                files[req_param] = params[req_param]
            else:
                clean_params[req_param] = params[req_param]

        for allowed_param in self.allowed_params:
            if allowed_param in params:
                if isinstance(params[allowed_param], bytes):
                    files[allowed_param] = params[allowed_param]
                else:
                    clean_params[allowed_param] = params[allowed_param]

        if self.drop_empty_params:
            clean_params = {k: v for k, v in clean_params.items() if ((v is not None) and (v != ""))}

        clean_params["files"] = files if files else None

        return clean_params

    def get_payload(self, params: dict) -> dict | list:
        """
        Получение body для POST запросов
        """

        if self.array_payload:
            return params["array_payload"]
        return params

    def get_request_params(self, **kwargs) -> dict:
        """
        Получение всех параметров, необходимых для запроса (url, headers, params, json, files)
        """

        request_params = {
            "url": self.build_url(kwargs),
            "headers": self.headers,
        }

        clean_params = self.get_clean_params(kwargs)

        files = clean_params.pop("files", None)
        if files:
            request_params["files"] = files

        if self.http_method in ["POST", "PUT", "PATCH"]:
            request_params["json"] = self.get_payload(params=clean_params)
        else:
            request_params["params"] = clean_params

        return request_params

    def make_request(self, **kwargs) -> requests.Response:
        """
        Непосредственный запрос к API
        """
        return requests.request(
            method=self.http_method,
            timeout=self.response_timeout,
            **self.get_request_params(**kwargs),
        )
